@@ -1,4 +1,4 @@
-#!/usr/bin/env python
+#!/usr/bin/env python2
 
 # THIS FILE IS PART OF THE CYLC SUITE ENGINE.
 # Copyright (C) 2008-2018 NIWA
@@ -38,8 +38,10 @@ from cylc.cfgspec.utils import (coerce_interval, DurationFloat)
 
 
 coercers['interval'] = coerce_interval
+OLD_SITE_FILE = os.path.join(
+    os.environ['CYLC_DIR'], 'etc', 'gcylcrc', 'themes.rc')
 SITE_FILE = os.path.join(
-    os.environ['CYLC_DIR'], 'conf', 'gcylcrc', 'themes.rc')
+    os.environ['CYLC_DIR'], 'etc', 'gcylc-themes.rc')
 USER_FILE = os.path.join(os.environ['HOME'], '.cylc', 'gcylc.rc')
 SPEC = {
     'dot icon size': vdr(
