@@ -1,4 +1,4 @@
-#!/usr/bin/env python
+#!/usr/bin/env python2
 
 # THIS FILE IS PART OF THE CYLC SUITE ENGINE.
 # Copyright (C) 2008-2018 NIWA
@@ -35,12 +35,14 @@ import cylc.flags
 from cylc.cfgspec.utils import (
     coerce_interval, coerce_interval_list, DurationFloat)
 from cylc.network import PRIVILEGE_LEVELS, PRIV_STATE_TOTALS, PRIV_SHUTDOWN
+from cylc.version import CYLC_VERSION
 
 coercers['interval'] = coerce_interval
 coercers['interval_list'] = coerce_interval_list
 
 SPEC = {
     'process pool size': vdr(vtype='integer', default=4),
+    'process pool timeout': vdr(vtype='interval', default=DurationFloat(600)),
     'temporary directory': vdr(vtype='string'),
     'state dump rolling archive length': vdr(
         vtype='integer', default=10),
@@ -166,22 +168,8 @@ SPEC = {
                 vtype='interval_list', default=[]),
             'task event handler retry delays': vdr(
                 vtype='interval_list', default=[]),
-            'local tail command template': vdr(
+            'tail command template': vdr(
                 vtype='string', default="tail -n +1 -F %(filename)s"),
-            'remote tail command template': vdr(
-                vtype='string',
-                default=(
-                    "tail --pid=`ps h -o ppid $$" +
-                    " | sed -e s/[[:space:]]//g` -n +1 -F %(filename)s")),
-            # Template for tail commands on remote files.  On signal to "ssh"
-            # client, a signal is sent to "sshd" on server.  However, "sshd"
-            # cannot send a signal to the "tail" command, because it is not a
-            # terminal. Apparently, we can use "ssh -t" or "ssh -tt", but that
-            # just causes the command to hang here for some reason. The easiest
-            # solution is to use the "--pid=PID" option of the "tail" command,
-            # so it dies as soon as PID dies. Note: if remote login shell is
-            # bash/ksh, we can use $PPID instead of `ps...` command, but we
-            # have to support login shell "tcsh" too.
             'batch systems': {
                 '__MANY__': {
                     'err tailer': vdr(vtype='string'),
@@ -217,8 +205,7 @@ SPEC = {
                 vtype='interval_list'),
             'task event handler retry delays': vdr(
                 vtype='interval_list'),
-            'local tail command template': vdr(vtype='string'),
-            'remote tail command template': vdr(vtype='string'),
+            'tail command template': vdr(vtype='string'),
             'batch systems': {
                 '__MANY__': {
                     'err tailer': vdr(vtype='string'),
@@ -250,6 +237,7 @@ SPEC = {
     'test battery': {
         'remote host with shared fs': vdr(vtype='string'),
         'remote host': vdr(vtype='string'),
+        'remote owner': vdr(vtype='string'),
         'batch systems': {
             '__MANY__': {
                 'host': vdr(vtype='string'),
@@ -379,6 +367,17 @@ def upg(cfg, descr):
         '7.3.1',
         ['hosts', '__MANY__', 'remote copy template'],
         ['hosts', '__MANY__', 'scp command'])
+    # A special remote tail command template is no longer needed. It used to
+    # use 'tail -pid' to kill 'tail' on ssh exit, but we do this in cylc now.
+    u.obsolete(
+        '7.6.0',
+        ['hosts', '__MANY__', 'remote tail command template']
+    )
+    u.deprecate(
+        '7.6.0',
+        ['hosts', '__MANY__', 'local tail command template'],
+        ['hosts', '__MANY__', 'tail command template']
+    )
     u.upgrade()
 
 
@@ -399,53 +398,67 @@ class GlobalConfig(config):
     """
 
     _DEFAULT = None
+    # For working with git clones, just go to minor version number.
+    cylc_version = re.sub('-.*', '', CYLC_VERSION)
     CONF_BASE = "global.rc"
-    SITE_CONF_DIR = os.path.join(os.environ["CYLC_DIR"], "conf")
-    USER_CONF_DIR = os.path.join(os.environ['HOME'], '.cylc')
-    OLD_SITE_CONF_BASE = os.path.join("siterc", "site.rc")
-    OLD_USER_CONF_BASE = os.path.join("user.rc")
+    # Site global.rc loc preference: if not in etc/ look in old conf/.
+    SITE_CONF_DIR = os.path.join(os.environ["CYLC_DIR"], "etc")
+    SITE_CONF_DIR_OLD = os.path.join(os.environ["CYLC_DIR"], "conf")
+    # User global.rc loc preference: if not in .cylc/x.y.z/ look in .cylc/.
+    USER_CONF_DIR_1 = os.path.join(os.environ['HOME'], '.cylc', cylc_version)
+    USER_CONF_DIR_2 = os.path.join(os.environ['HOME'], '.cylc')
 
     @classmethod
     def get_inst(cls):
         """Return the singleton instance."""
         if not cls._DEFAULT:
-            if cylc.flags.verbose:
-                print "Loading site/user config files"
             cls._DEFAULT = cls(SPEC, upg)
-            conf_path_str = os.getenv("CYLC_CONF_PATH")
-            count = 0
-            if conf_path_str is None:
-                # CYLC_CONF_PATH not defined, use default locations
-                for old_base, conf_dir in [
-                        [cls.OLD_SITE_CONF_BASE, cls.SITE_CONF_DIR],
-                        [cls.OLD_USER_CONF_BASE, cls.USER_CONF_DIR]]:
-                    for base in [cls.CONF_BASE, old_base]:
-                        file_name = os.path.join(conf_dir, base)
-                        if os.access(file_name, os.F_OK | os.R_OK):
-                            try:
-                                cls._DEFAULT.loadcfg(
-                                    file_name, "global config")
-                            except ParsecError as exc:
-                                if count == 0:
-                                    sys.stderr.write(
-                                        "WARNING: ignoring bad site config %s:"
-                                        "\n%s\n" % (file_name, str(exc)))
-                                else:
-                                    sys.stderr.write(
-                                        "ERROR: bad user config %s:\n" % (
-                                            file_name))
-                                    raise
-                            count += 1
-                            break
-            elif conf_path_str:
-                # CYLC_CONF_PATH defined with a value
-                for path in conf_path_str.split(os.pathsep):
-                    file_name = os.path.join(path, cls.CONF_BASE)
-                    if os.access(file_name, os.F_OK | os.R_OK):
-                        cls._DEFAULT.loadcfg(file_name, "global config")
-            cls._DEFAULT.transform()
+            cls._DEFAULT.load()
         return cls._DEFAULT
 
+    def load(self):
+        """Load or reload configuration from files."""
+        self.sparse.clear()
+        self.dense.clear()
+        if cylc.flags.verbose:
+            print "Loading site/user global config files"
+        conf_path_str = os.getenv("CYLC_CONF_PATH")
+        if conf_path_str is None:
+            # CYLC_CONF_PATH not defined, use default locations.
+            for conf_dir_1, conf_dir_2, is_site in [
+                    (self.SITE_CONF_DIR, self.SITE_CONF_DIR_OLD, True),
+                    (self.USER_CONF_DIR_1, self.USER_CONF_DIR_2, False)]:
+                fname1 = os.path.join(conf_dir_1, self.CONF_BASE)
+                fname2 = os.path.join(conf_dir_2, self.CONF_BASE)
+                if os.access(fname1, os.F_OK | os.R_OK):
+                    fname = fname1
+                elif os.access(fname2, os.F_OK | os.R_OK):
+                    fname = fname2
+                else:
+                    continue
+                try:
+                    self.loadcfg(fname, "global config")
+                except ParsecError as exc:
+                    if is_site:
+                        # Warn on bad site file (users can't fix it).
+                        sys.stderr.write(
+                            "WARNING: ignoring bad site config %s:"
+                            "\n%s\n" % (fname, str(exc)))
+                    else:
+                        # Abort on bad user file (users can fix it).
+                        sys.stderr.write(
+                            "ERROR: bad user config %s:\n" % (fname))
+                        raise
+                    break
+        elif conf_path_str:
+            # CYLC_CONF_PATH defined with a value
+            for path in conf_path_str.split(os.pathsep):
+                fname = os.path.join(path, self.CONF_BASE)
+                if os.access(fname, os.F_OK | os.R_OK):
+                    self.loadcfg(fname, "global config")
+        # (OK if no global.rc is found, just use system defaults).
+        self.transform()
+
     def get_derived_host_item(
             self, suite, item, host=None, owner=None, replace_home=False):
         """Compute hardwired paths relative to the configurable top dirs."""
@@ -648,6 +661,3 @@ class GlobalConfig(config):
         for key, val in cfg['hosts']['localhost'].items():
             if val and 'directory' in key:
                 cfg['hosts']['localhost'][key] = expandvars(val)
-
-
-GLOBAL_CFG = GlobalConfig.get_inst()
