@@ -54,6 +54,7 @@ class SuiteSpecifics(object):
     abbrev_util = None
     interval_parser = None
     point_parser = None
+    recurrence_parser = None
     iso8601_parsers = None
 
 
@@ -294,11 +295,8 @@ class ISO8601Exclusions(ExclusionBase):
     may be used on this object to determine if a point is in the collection
     of exclusion sequences."""
 
-    __slots__ = ExclusionBase.__slots__ + ('p_iso_exclusions',)
-
     def __init__(self, excl_points, start_point, end_point=None):
         super(ISO8601Exclusions, self).__init__(start_point, end_point)
-        self.p_iso_exclusions = []
         self.build_exclusions(excl_points)
 
     def build_exclusions(self, excl_points):
@@ -309,7 +307,6 @@ class ISO8601Exclusions(ExclusionBase):
                     str(point)) if point else None
                 if exclusion_point not in self.exclusion_points:
                     self.exclusion_points.append(exclusion_point)
-                    self.p_iso_exclusions.append(str(exclusion_point))
             except (AttributeError, TypeError, ValueError):
                 # Try making an ISO8601Sequence
                 exclusion = ISO8601Sequence(point, self.exclusion_start_point,
@@ -487,29 +484,29 @@ class ISO8601Sequence(SequenceBase):
         if self.is_on_sequence(point):
             return self.get_prev_point(point)
         p_iso_point = point_parse(point.value)
-        prev_iso_point = None
+        prev_cycle_point = None
 
         for recurrence_iso_point in self.recurrence:
+
             # Is recurrence point greater than aribitrary point?
-            if (
-                    recurrence_iso_point > p_iso_point or
-                    (self.exclusions and
-                     recurrence_iso_point in self.exclusions.p_iso_exclusions)
-            ):
+            if recurrence_iso_point > p_iso_point:
                 break
-            prev_iso_point = recurrence_iso_point
-        if prev_iso_point is None:
+            recurrence_cycle_point = ISO8601Point(str(recurrence_iso_point))
+            if self.exclusions and recurrence_cycle_point in self.exclusions:
+                break
+            prev_cycle_point = recurrence_cycle_point
+
+        if prev_cycle_point is None:
             return None
-        nearest_point = ISO8601Point(str(prev_iso_point))
-        if nearest_point == point:
+        if prev_cycle_point == point:
             raise SequenceDegenerateError(
                 self.recurrence, SuiteSpecifics.DUMP_FORMAT,
-                nearest_point, point
+                prev_cycle_point, point
             )
         # Check all exclusions
-        if self.exclusions and nearest_point in self.exclusions:
-            return self.get_prev_point(nearest_point)
-        return nearest_point
+        if self.exclusions and prev_cycle_point in self.exclusions:
+            return self.get_prev_point(prev_cycle_point)
+        return prev_cycle_point
 
     def get_next_point(self, point):
         """Return the next point > p, or None if out of bounds."""
@@ -672,9 +669,6 @@ def init_from_cfg(cfg):
 def init(num_expanded_year_digits=0, custom_dump_format=None, time_zone=None,
          assume_utc=False, cycling_mode=None):
     """Initialise suite-setup-specific information."""
-
-    SuiteSpecifics.interval_parser = DurationParser()
-
     if cycling_mode in Calendar.default().MODES:
         Calendar.default().set_mode(cycling_mode)
 
@@ -702,13 +696,6 @@ def init(num_expanded_year_digits=0, custom_dump_format=None, time_zone=None,
                 ('cylc', 'cycle point format'),
                 SuiteSpecifics.DUMP_FORMAT
             )
-    SuiteSpecifics.point_parser = TimePointParser(
-        allow_only_basic=False,
-        allow_truncated=True,
-        num_expanded_year_digits=SuiteSpecifics.NUM_EXPANDED_YEAR_DIGITS,
-        dump_format=SuiteSpecifics.DUMP_FORMAT,
-        assumed_time_zone=time_zone_hours_minutes
-    )
 
     SuiteSpecifics.iso8601_parsers = CylcTimeParser.initiate_parsers(
         dump_format=SuiteSpecifics.DUMP_FORMAT,
@@ -716,6 +703,10 @@ def init(num_expanded_year_digits=0, custom_dump_format=None, time_zone=None,
         assumed_time_zone=SuiteSpecifics.ASSUMED_TIME_ZONE
     )
 
+    (SuiteSpecifics.point_parser,
+     SuiteSpecifics.interval_parser,
+     SuiteSpecifics.recurrence_parser) = SuiteSpecifics.iso8601_parsers
+
     SuiteSpecifics.abbrev_util = CylcTimeParser(
         None, None, SuiteSpecifics.iso8601_parsers
     )
