@@ -1,4 +1,4 @@
-#!/usr/bin/env python
+#!/usr/bin/env python2
 
 # THIS FILE IS PART OF THE CYLC SUITE ENGINE.
 # Copyright (C) 2008-2018 NIWA
@@ -21,7 +21,7 @@
 import os
 import sys
 from time import sleep, time
-from cylc.suite_logging import SuiteLog
+from cylc.suite_logging import SuiteLog, SUITE_LOG
 
 
 SUITE_SCAN_INFO_TMPL = r"""
@@ -30,7 +30,7 @@ To view suite server program contact information:
  $ cylc get-suite-contact %(suite)s
 
 Other ways to see if the suite is still running:
- $ cylc scan -n '\b%(suite)s\b' %(host)s
+ $ cylc scan -n '%(suite)s' %(host)s
  $ cylc ping -v --host=%(host)s %(suite)s
  $ ps %(ps_opts)s %(pid)s  # on %(host)s
 
@@ -53,7 +53,7 @@ def daemonize(server):
 
     """
     logd = SuiteLog.get_dir_for_suite(server.suite)
-    log_fname = os.path.join(logd, SuiteLog.LOG)
+    log_fname = os.path.join(logd, SUITE_LOG)
     try:
         old_log_mtime = os.stat(log_fname).st_mtime
     except OSError:
