@@ -1,4 +1,4 @@
-#!/usr/bin/env python
+#!/usr/bin/env python2
 
 # THIS FILE IS PART OF THE CYLC SUITE ENGINE.
 # Copyright (C) 2008-2018 NIWA
@@ -27,7 +27,7 @@ def check_varnames(env):
     returns a list of bad names (empty implies success)."""
     bad = []
     for varname in env:
-        if not re.match('^[a-zA-Z_][\w]*$', varname):
+        if not re.match(r'^[a-zA-Z_][\w]*$', varname):
             bad.append(varname)
     return bad
 
