@@ -1,4 +1,4 @@
-#!/usr/bin/env python
+#!/usr/bin/env python2
 
 # THIS FILE IS PART OF THE CYLC SUITE ENGINE.
 # Copyright (C) 2008-2018 NIWA
@@ -17,7 +17,6 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 import gtk
-import gobject
 
 from cylc.gui.updater_graph import GraphUpdater
 from cylc.cylc_xdot import xdot_widgets
@@ -109,7 +108,11 @@ Dependency graph suite control interface.
             # base graph node
             task_id = task_id[len(self.t.PREFIX_BASE):]
 
-        name, point_string = TaskID.split(task_id)
+        try:
+            name, point_string = TaskID.split(task_id)
+        except ValueError:
+            # not a task -> skip
+            return False
 
         menu = gtk.Menu()
         menu_root = gtk.MenuItem(task_id)
