@@ -1,4 +1,4 @@
-#!/usr/bin/env python
+#!/usr/bin/env python2
 
 # THIS FILE IS PART OF THE CYLC SUITE ENGINE.
 # Copyright (C) 2008-2018 NIWA
@@ -23,7 +23,7 @@ import stat
 from subprocess import Popen, PIPE
 
 from cylc.batch_sys_manager import BatchSysManager
-from cylc.cfgspec.globalcfg import GLOBAL_CFG
+from cylc.cfgspec.glbl_cfg import glbl_cfg
 import cylc.flags
 
 
@@ -115,14 +115,14 @@ class JobFileWriter(object):
 
     @staticmethod
     def _get_derived_host_item(job_conf, key):
-        """Return derived host item from GLOBAL_CFG."""
-        return GLOBAL_CFG.get_derived_host_item(
+        """Return derived host item from glbl_cfg()."""
+        return glbl_cfg().get_derived_host_item(
             job_conf['suite_name'], key, job_conf["host"], job_conf["owner"])
 
     @staticmethod
     def _get_host_item(job_conf, key):
-        """Return host item from GLOBAL_CFG."""
-        return GLOBAL_CFG.get_host_item(
+        """Return host item from glbl_cfg()."""
+        return glbl_cfg().get_host_item(
             key, job_conf["host"], job_conf["owner"])
 
     @staticmethod
@@ -201,12 +201,17 @@ class JobFileWriter(object):
         handle.write(
             '\n    export CYLC_SUITE_DEF_PATH_ON_SUITE_HOST="%s"' %
             os.environ['CYLC_SUITE_DEF_PATH'])
+        handle.write(
+            '\n    export CYLC_SUITE_UUID="%s"' % job_conf['uuid_str'])
 
         handle.write("\n\n    # CYLC TASK ENVIRONMENT:")
         handle.write('\n    export CYLC_TASK_JOB="%s"' % job_conf['job_d'])
         handle.write(
             '\n    export CYLC_TASK_NAMESPACE_HIERARCHY="%s"' %
             ' '.join(job_conf['namespace_hierarchy']))
+        handle.write(
+            '\n    export CYLC_TASK_DEPENDENCIES="%s"' %
+            ' '.join(job_conf['dependencies']))
         handle.write(
             '\n    export CYLC_TASK_TRY_NUMBER=%s' % job_conf['try_num'])
         # Custom parameter environment variables
@@ -222,18 +227,6 @@ class JobFileWriter(object):
                 "\n    CYLC_TASK_WORK_DIR_BASE='%s'" % job_conf['work_d'])
         handle.write("\n}")
 
-        # SSH comms variables. Note:
-        # For "poll", contact file will not be installed, and job will not
-        # attempt to communicate back.
-        # Otherwise, job will attempt to communicate back via HTTP(S).
-        comms = self._get_host_item(job_conf, 'task communication method')
-        if comms == 'ssh':
-            handle.write("\n\n    # CYLC MESSAGE ENVIRONMENT:")
-            handle.write('\n    export CYLC_TASK_COMMS_METHOD="%s"' % comms)
-            handle.write(
-                '\n    export CYLC_TASK_SSH_LOGIN_SHELL="%s"' %
-                self._get_host_item(job_conf, 'use login shell'))
-
     @staticmethod
     def _write_environment_2(handle, job_conf):
         """Run time environment part 2."""
