@@ -1,4 +1,4 @@
-#!/usr/bin/env python
+#!/usr/bin/env python2
 
 # THIS FILE IS PART OF THE CYLC SUITE ENGINE.
 # Copyright (C) 2008-2018 NIWA
@@ -25,12 +25,13 @@ import binascii
 import inspect
 import os
 import random
+import re
 from time import time
 import traceback
 from uuid import uuid4
 
 import cherrypy
-from cylc.cfgspec.globalcfg import GLOBAL_CFG
+from cylc.cfgspec.glbl_cfg import glbl_cfg
 from cylc.exceptions import CylcError
 import cylc.flags
 from cylc.network import (
@@ -42,12 +43,13 @@ from cylc.suite_srv_files_mgr import (
     SuiteSrvFilesManager, SuiteServiceFileError)
 from cylc.unicode_util import utf8_enforce
 from cylc.version import CYLC_VERSION
+from cylc.wallclock import RE_DATE_TIME_FORMAT_EXTENDED
 
 
 class HTTPServer(object):
     """HTTP(S) server by cherrypy, for serving suite runtime API."""
 
-    API = 1
+    API = 2
     LOG_CONNECT_DENIED_TMPL = "[client-connect] DENIED %s@%s:%s %s"
 
     def __init__(self, suite):
@@ -57,13 +59,13 @@ class HTTPServer(object):
         self.port = None
 
         # Figure out the ports we are allowed to use.
-        base_port = GLOBAL_CFG.get(['communication', 'base port'])
-        max_ports = GLOBAL_CFG.get(
+        base_port = glbl_cfg().get(['communication', 'base port'])
+        max_ports = glbl_cfg().get(
             ['communication', 'maximum number of ports'])
         self.ok_ports = range(int(base_port), int(base_port) + int(max_ports))
         random.shuffle(self.ok_ports)
 
-        comms_options = GLOBAL_CFG.get(['communication', 'options'])
+        comms_options = glbl_cfg().get(['communication', 'options'])
 
         # HTTP Digest Auth uses MD5 - pretty secure in this use case.
         # Extending it with extra algorithms is allowed, but won't be
@@ -74,7 +76,7 @@ class HTTPServer(object):
             self.hash_algorithm = "SHA"
 
         self.srv_files_mgr = SuiteSrvFilesManager()
-        self.comms_method = GLOBAL_CFG.get(['communication', 'method'])
+        self.comms_method = glbl_cfg().get(['communication', 'method'])
         self.get_ha1 = cherrypy.lib.auth_digest.get_ha1_dict_plain(
             {
                 'cylc': self.srv_files_mgr.get_auth_item(
@@ -200,6 +202,8 @@ class SuiteRuntimeService(object):
     LOG_IDENTIFY_TMPL = '[client-identify] %d id requests in PT%dS'
     LOG_FORGET_TMPL = '[client-forget] %s'
     LOG_CONNECT_ALLOWED_TMPL = "[client-connect] %s@%s:%s privilege='%s' %s"
+    RE_MESSAGE_TIME = re.compile(
+        r'\A(.+) at (' + RE_DATE_TIME_FORMAT_EXTENDED + r')\Z', re.DOTALL)
 
     def __init__(self, schd):
         self.schd = schd
@@ -498,10 +502,45 @@ class SuiteRuntimeService(object):
     @cherrypy.expose
     @cherrypy.tools.json_out()
     def put_message(self, task_id, severity, message):
+        """(Compat) Put task message.
+
+        Arguments:
+            task_id (str): Task ID in the form "TASK_NAME.CYCLE".
+            severity (str): Severity level of message.
+            message (str): Content of message.
+        """
         self._check_access_priv_and_report(PRIV_FULL_CONTROL, log_info=False)
-        self.schd.message_queue.put((task_id, severity, str(message)))
+        match = self.RE_MESSAGE_TIME.match(message)
+        event_time = None
+        if match:
+            message, event_time = match.groups()
+        self.schd.message_queue.put(
+            (task_id, event_time, severity, message))
         return (True, 'Message queued')
 
+    @cherrypy.expose
+    @cherrypy.tools.json_in()
+    @cherrypy.tools.json_out()
+    def put_messages(self, task_job=None, event_time=None, messages=None):
+        """Put task messages in queue for processing later by the main loop.
+
+        Arguments:
+            task_job (str): Task job in the form "CYCLE/TASK_NAME/SUBMIT_NUM".
+            event_time (str): Event time as string.
+            messages (list): List in the form [[severity, message], ...].
+        """
+        self._check_access_priv_and_report(PRIV_FULL_CONTROL, log_info=False)
+        task_job = utf8_enforce(
+            cherrypy.request.json.get("task_job", task_job))
+        event_time = utf8_enforce(
+            cherrypy.request.json.get("event_time", event_time))
+        messages = utf8_enforce(
+            cherrypy.request.json.get("messages", messages))
+        for severity, message in messages:
+            self.schd.message_queue.put(
+                (task_job, event_time, severity, message))
+        return (True, 'Messages queued: %d' % len(messages))
+
     @cherrypy.expose
     @cherrypy.tools.json_out()
     def reload_suite(self):
@@ -684,7 +723,7 @@ class SuiteRuntimeService(object):
         return (True, 'Command queued')
 
     def _access_priv_ok(self, required_privilege_level):
-        """Return True if a client is allowed access to info from server_obj.
+        """Return True if a client has enough privilege for given level.
 
         The required privilege level is compared to the level granted to the
         client by the connection validator (held in thread local storage).
@@ -696,7 +735,7 @@ class SuiteRuntimeService(object):
             return False
 
     def _check_access_priv(self, required_privilege_level):
-        """Raise an exception if client privilege is insufficient for server_obj.
+        """Raise an exception if client privilege is insufficient.
 
         (See the documentation above for the boolean version of this function).
 
