@@ -1,4 +1,4 @@
-#!/usr/bin/env python
+#!/usr/bin/env python2
 
 # THIS FILE IS PART OF THE CYLC SUITE ENGINE.
 # Copyright (C) 2008-2018 NIWA
@@ -126,10 +126,8 @@ Arguments:"""
             action="store_true", default=False, dest="verbose")
         self.add_std_option(
             "--debug",
-            help=(
-                "Run suites in non-daemon mode, "
-                "and show exception tracebacks."
-            ),
+            help=("Output developer information and show exception "
+                  "tracebacks."),
             action="store_true", default=False, dest="debug")
 
         if self.prep:
@@ -152,6 +150,10 @@ Arguments:"""
                 "--use-ssh",
                 help="Use ssh to re-invoke the command on the suite host.",
                 action="store_true", default=False, dest="use_ssh")
+            self.add_std_option(
+                "--ssh-cylc",
+                help="Location of cylc executable on remote ssh comamnds.",
+                action="store", default="cylc", dest="ssh_cylc")
             self.add_std_option(
                 "--no-login",
                 help=(
