@@ -1,4 +1,4 @@
-#!/usr/bin/env python
+#!/usr/bin/env python2
 
 # THIS FILE IS PART OF THE CYLC SUITE ENGINE.
 # Copyright (C) 2008-2018 NIWA
@@ -25,7 +25,7 @@ a_hbar = '-'
 a_vbar = '|'
 a_tee = a_vbar + a_hbar
 a_trm = '`' + a_hbar
-a_tee_re = '\|' + a_hbar
+a_tee_re = r'\|' + a_hbar
 
 # Unicode box-printing characters
 u_hbar = u'\u2500'
