@@ -1,4 +1,4 @@
-#!/usr/bin/env python
+#!/usr/bin/env python2
 
 # THIS FILE IS PART OF THE CYLC SUITE ENGINE.
 # Copyright (C) 2008-2018 NIWA
@@ -17,13 +17,14 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 """Provide data access object for the suite runtime database."""
 
+import json
 import sqlite3
 import sys
 import traceback
 
 import cylc.flags
 from cylc.wallclock import get_current_time_string
-from cylc.suite_logging import LOG, ERR
+from cylc.suite_logging import LOG
 
 
 class CylcSuiteDAOTableColumn(object):
@@ -186,6 +187,7 @@ class CylcSuiteDAO(object):
     TABLE_TASK_EVENTS = "task_events"
     TABLE_TASK_ACTION_TIMERS = "task_action_timers"
     TABLE_CHECKPOINT_ID = "checkpoint_id"
+    TABLE_TASK_LATE_FLAGS = "task_late_flags"
     TABLE_TASK_OUTPUTS = "task_outputs"
     TABLE_TASK_POOL = "task_pool"
     TABLE_TASK_POOL_CHECKPOINTS = "task_pool_checkpoints"
@@ -239,9 +241,9 @@ class CylcSuiteDAO(object):
         TABLE_TASK_ACTION_TIMERS: [
             ["cycle", {"is_primary_key": True}],
             ["name", {"is_primary_key": True}],
-            ["ctx_key_pickle", {"is_primary_key": True}],
-            ["ctx_pickle"],
-            ["delays_pickle"],
+            ["ctx_key", {"is_primary_key": True}],
+            ["ctx"],
+            ["delays"],
             ["num", {"datatype": "INTEGER"}],
             ["delay"],
             ["timeout"],
@@ -271,6 +273,11 @@ class CylcSuiteDAO(object):
             ["event"],
             ["message"],
         ],
+        TABLE_TASK_LATE_FLAGS: [
+            ["cycle", {"is_primary_key": True}],
+            ["name", {"is_primary_key": True}],
+            ["value", {"datatype": "INTEGER"}],
+        ],
         TABLE_TASK_OUTPUTS: [
             ["cycle", {"is_primary_key": True}],
             ["name", {"is_primary_key": True}],
@@ -461,7 +468,7 @@ class CylcSuiteDAO(object):
             for i, stmt_args in enumerate(stmt_args_list):
                 err_log += ("\nstmt_args[%(i)d]=%(stmt_args)s" % {
                     "i": i, "stmt_args": stmt_args})
-            ERR.warning(err_log)
+            LOG.warning(err_log)
             raise
 
     def select_broadcast_states(self, callback, id_key=None):
@@ -533,6 +540,16 @@ class CylcSuiteDAO(object):
                 r"SELECT key,value FROM %s" % self.TABLE_SUITE_TEMPLATE_VARS)):
             callback(row_idx, list(row))
 
+    def select_table_schema(self, my_type, my_name):
+        """Select from task_action_timers for restart.
+
+        Invoke callback(row_idx, row) on each row.
+        """
+        for sql, in self.connect().execute(
+                r"SELECT sql FROM sqlite_master where type==? and name==?",
+                [my_type, my_name]):
+            return sql
+
     def select_task_action_timers(self, callback):
         """Select from task_action_timers for restart.
 
@@ -653,8 +670,8 @@ class CylcSuiteDAO(object):
         """Select from task_pool+task_states+task_jobs for restart.
 
         Invoke callback(row_idx, row) on each row, where each row contains:
-            [cycle, name, spawned, status, hold_swap, submit_num, try_num,
-             user_at_host, time_submit, time_run, timeout]
+            [cycle, name, spawned, is_late, status, hold_swap, submit_num,
+             try_num, user_at_host, time_submit, time_run, timeout, outputs]
 
         If id_key is specified,
         select from task_pool table if id_key == CHECKPOINT_LATEST_ID.
@@ -665,6 +682,7 @@ class CylcSuiteDAO(object):
                 %(task_pool)s.cycle,
                 %(task_pool)s.name,
                 %(task_pool)s.spawned,
+                %(task_late_flags)s.value,
                 %(task_pool)s.status,
                 %(task_pool)s.hold_swap,
                 %(task_states)s.submit_num,
@@ -680,6 +698,10 @@ class CylcSuiteDAO(object):
                 %(task_states)s
             ON  %(task_pool)s.cycle == %(task_states)s.cycle AND
                 %(task_pool)s.name == %(task_states)s.name
+            LEFT OUTER JOIN
+                %(task_late_flags)s
+            ON  %(task_pool)s.cycle == %(task_late_flags)s.cycle AND
+                %(task_pool)s.name == %(task_late_flags)s.name
             LEFT OUTER JOIN
                 %(task_jobs)s
             ON  %(task_pool)s.cycle == %(task_jobs)s.cycle AND
@@ -697,6 +719,7 @@ class CylcSuiteDAO(object):
         form_data = {
             "task_pool": self.TABLE_TASK_POOL,
             "task_states": self.TABLE_TASK_STATES,
+            "task_late_flags": self.TABLE_TASK_LATE_FLAGS,
             "task_timeout_timers": self.TABLE_TASK_TIMEOUT_TIMERS,
             "task_jobs": self.TABLE_TASK_JOBS,
             "task_outputs": self.TABLE_TASK_OUTPUTS,
@@ -813,7 +836,7 @@ class CylcSuiteDAO(object):
                 # run mode, time stamp, initial cycle, final cycle
                 location = self._upgrade_with_state_file_header(line)
             elif location == "broadcast":
-                # Ignore broadcast pickle in state file.
+                # Ignore broadcast json in state file.
                 # The "broadcast_states" table should already be populated.
                 if line == "Begin task states":
                     location = "task states"
@@ -904,6 +927,75 @@ class CylcSuiteDAO(object):
             conn.execute(r"DROP TABLE " + t_name + "_old")
         conn.commit()
 
+    def upgrade_pickle_to_json(self):
+        """Upgrade the database tables if containing pickled objects.
+
+        Back compat for <=7.6.X.
+        """
+        conn = self.connect()
+        t_name = self.TABLE_TASK_ACTION_TIMERS
+        if "_pickle" not in self.select_table_schema("table", t_name):
+            return
+
+        # Rename old tables
+        conn.execute(r"ALTER TABLE %(table)s RENAME TO %(table)s_old" % {
+            "table": t_name})
+        conn.commit()
+
+        # Create tables with new columns
+        self.create_tables()
+
+        # Populate new tables using old column data
+        # Codacy: Pickle library appears to be in use, possible security issue.
+        # Use of "pickle" module is for loading data written by <=7.6.X of Cylc
+        # in users' own spaces.
+        import pickle
+        sys.stdout.write(r"Upgrading %s table " % (t_name))
+        cols = []
+        for col in self.tables[t_name].columns:
+            if col.name in ['ctx_key', 'ctx', 'delays']:
+                cols.append(col.name + '_pickle')
+            else:
+                cols.append(col.name)
+        n_skips = 0
+        # Codacy: Possible SQL injection vector through string-based query
+        # construction.
+        # This is highly unlikely - all strings in the constuct are from
+        # constants in this module.
+        for i, row in enumerate(conn.execute(
+                r"SELECT " + ",".join(cols) + " FROM " + t_name + "_old")):
+            args = []
+            try:
+                for col, cell in zip(cols, row):
+                    if col == "ctx_pickle":
+                        # Upgrade pickled namedtuple objects
+                        orig = pickle.loads(str(cell))
+                        if orig is not None:
+                            args.append(json.dumps(
+                                [type(orig).__name__, orig.__getnewargs__()]))
+                        else:
+                            args.append(json.dumps(orig))
+                    elif col.endswith("_pickle"):
+                        # Upgrade pickled lists
+                        args.append(json.dumps(pickle.loads(str(cell))))
+                    else:
+                        args.append(cell)
+            except (EOFError, TypeError, LookupError, ValueError):
+                n_skips += 1  # skip bad rows
+            else:
+                # These tables can be big, so we don't want to queue the items
+                # in memory.
+                conn.execute(self.tables[t_name].get_insert_stmt(), args)
+                if i:
+                    sys.stdout.write("\b" * len("%d rows" % (i)))
+                sys.stdout.write("%d rows" % (i + 1))
+        sys.stdout.write(" done, %d skipped\n" % n_skips)
+        conn.commit()
+
+        # Drop old tables
+        conn.execute(r"DROP TABLE %(table)s_old" % {"table": t_name})
+        conn.commit()
+
     def vacuum(self):
         """Vacuum to the database."""
         return self.connect().execute("VACUUM")
