@@ -1,4 +1,4 @@
-#!/usr/bin/env python
+#!/usr/bin/env python2
 
 # THIS FILE IS PART OF THE CYLC SUITE ENGINE.
 # Copyright (C) 2008-2018 NIWA
@@ -31,7 +31,7 @@ import isodatetime.data
 import isodatetime.parsers
 from parsec.util import printcfg
 
-from cylc.cfgspec.globalcfg import GLOBAL_CFG
+from cylc.cfgspec.glbl_cfg import glbl_cfg
 from cylc.config import SuiteConfig
 from cylc.cycling import PointParsingError
 from cylc.cycling.loader import get_point, standardise_point_string
@@ -44,10 +44,11 @@ from cylc.network import PRIVILEGE_LEVELS
 from cylc.network.httpserver import HTTPServer
 from cylc.state_summary_mgr import StateSummaryMgr
 from cylc.suite_db_mgr import SuiteDatabaseManager
+from cylc.task_job_logs import JOB_LOG_JOB, get_task_job_log
 from cylc.suite_events import (
     SuiteEventContext, SuiteEventError, SuiteEventHandler)
 from cylc.hostuserutil import get_host, get_user
-from cylc.suite_logging import SuiteLog, ERR, LOG
+from cylc.suite_logging import SuiteLog, SUITE_LOG, SUITE_ERR, ERR, LOG
 from cylc.suite_srv_files_mgr import (
     SuiteSrvFilesManager, SuiteServiceFileError)
 from cylc.suite_status import (
@@ -58,11 +59,13 @@ from cylc.task_id import TaskID
 from cylc.task_job_mgr import TaskJobManager
 from cylc.task_pool import TaskPool
 from cylc.task_proxy import TaskProxy, TaskProxySequenceBoundsError
-from cylc.task_state import TASK_STATUSES_ACTIVE, TASK_STATUS_FAILED
+from cylc.task_state import (
+    TASK_STATUSES_ACTIVE, TASK_STATUSES_NEVER_ACTIVE, TASK_STATUS_FAILED)
 from cylc.templatevars import load_template_vars
 from cylc.version import CYLC_VERSION
 from cylc.wallclock import (
-    get_current_time_string, get_seconds_as_interval_string)
+    get_current_time_string, get_seconds_as_interval_string,
+    get_time_string_from_unix_time as time2str)
 from cylc.profiler import Profiler
 
 
@@ -87,6 +90,7 @@ class Scheduler(object):
 
     # Intervals in seconds
     INTERVAL_MAIN_LOOP = 1.0
+    INTERVAL_MAIN_LOOP_QUICK = 0.5
     INTERVAL_STOP_KILL = 10.0
     INTERVAL_STOP_PROCESS_POOL_EMPTY = 0.5
 
@@ -126,22 +130,20 @@ class Scheduler(object):
         self.suiterc_update_time = None
         # For user-defined batch system handlers
         sys.path.append(os.path.join(self.suite_dir, 'python'))
-        self.suite_run_dir = GLOBAL_CFG.get_derived_host_item(
+        self.suite_run_dir = glbl_cfg().get_derived_host_item(
             self.suite, 'suite run directory')
         self.config = None
 
         self.is_restart = is_restart
-        if self.is_restart:
-            self.restart_warm_point = None
-        self._cli_initial_point_string = None
-        self._cli_start_point_string = None
+        self.cli_initial_point_string = None
+        self.cli_start_point_string = None
         start_point_str = None
         if len(args) > 1:
             start_point_str = args[1]
         if getattr(self.options, 'warm', None):
-            self._cli_start_point_string = start_point_str
+            self.cli_start_point_string = start_point_str
         else:
-            self._cli_initial_point_string = start_point_str
+            self.cli_initial_point_string = start_point_str
         self.template_vars = load_template_vars(
             self.options.templatevars, self.options.templatevars_file)
 
@@ -149,7 +151,6 @@ class Scheduler(object):
 
         self.owner = get_user()
         self.host = get_host()
-        self.port = None
 
         self.is_stalled = False
 
@@ -164,6 +165,7 @@ class Scheduler(object):
         self.task_events_mgr = None
         self.suite_event_handler = None
         self.httpserver = None
+        self.port = None
         self.command_queue = None
         self.message_queue = None
         self.ext_trigger_queue = None
@@ -210,7 +212,7 @@ class Scheduler(object):
         """Start the server."""
         self._start_print_blurb()
 
-        GLOBAL_CFG.create_cylc_run_tree(self.suite)
+        glbl_cfg().create_cylc_run_tree(self.suite)
 
         if self.is_restart:
             self.suite_db_mgr.restart_upgrade()
@@ -222,10 +224,11 @@ class Scheduler(object):
                 daemonize(self)
 
             # Setup the suite log.
-            SuiteLog.get_inst(self.suite).pimp(detach)
-
+            self.suite_log = SuiteLog.get_inst(self.suite)
+            self.suite_log.pimp(detach)
             self.proc_pool = SuiteProcPool()
-            self.configure_comms_daemon()
+            self.httpserver = HTTPServer(self.suite)
+            self.port = self.httpserver.port
             self.configure()
             self.profiler.start()
             self.run()
@@ -299,7 +302,6 @@ conditions; see `cylc conditions`.
         self.profiler.log_memory("scheduler.py: start configure")
 
         # Start up essential services
-        self.suite_log = SuiteLog.get_inst(self.suite)
         self.state_summary_mgr = StateSummaryMgr()
         self.command_queue = Queue()
         self.message_queue = Queue()
@@ -314,13 +316,16 @@ conditions; see `cylc conditions`.
             # This logic handles the lack of initial cycle point in "suite.rc".
             # Things that can't change on suite reload.
             pri_dao = self.suite_db_mgr.get_pri_dao()
-            pri_dao.select_suite_params(self._load_initial_cycle_point)
+            pri_dao.select_suite_params(self._load_suite_params_1)
+            # Configure contact data only after loading UUID string
+            self.configure_contact()
             pri_dao.select_suite_template_vars(self._load_template_vars)
-            pri_dao.select_suite_params(self._load_warm_cycle_point)
             # Take checkpoint and commit immediately so that checkpoint can be
             # copied to the public database.
             pri_dao.take_checkpoints("restart")
             pri_dao.execute_queued_items()
+        else:
+            self.configure_contact()
 
         self.profiler.log_memory("scheduler.py: before load_suiterc")
         self.load_suiterc()
@@ -356,7 +361,8 @@ conditions; see `cylc conditions`.
             self.configure_reftest()
 
         LOG.info(self.START_MESSAGE_TMPL % {
-            'host': self.host, 'port': self.port, 'pid': os.getpid()})
+            'host': self.host, 'port': self.httpserver.port,
+            'pid': os.getpid()})
         # Note that the following lines must be present at the top of
         # the suite log file for use in reference test runs:
         LOG.info('Cylc version: %s' % CYLC_VERSION)
@@ -377,13 +383,7 @@ conditions; see `cylc conditions`.
             self.load_tasks_for_run()
         self.profiler.log_memory("scheduler.py: after load_tasks")
 
-        self.suite_db_mgr.put_suite_params(
-            self.run_mode,
-            self.initial_point,
-            self.final_point,
-            self.pool.is_held,
-            self.config.cfg['cylc']['cycle point format'],
-            self._cli_start_point_string)
+        self.suite_db_mgr.put_suite_params(self)
         self.suite_db_mgr.put_suite_template_vars(self.template_vars)
         self.suite_db_mgr.put_runtime_inheritance(self.config)
         self.configure_suite_environment()
@@ -437,9 +437,9 @@ conditions; see `cylc conditions`.
     def load_tasks_for_restart(self):
         """Load tasks for restart."""
         self.suite_db_mgr.pri_dao.select_suite_params(
-            self._load_suite_params, self.options.checkpoint)
-        if self.restart_warm_point:
-            self.start_point = self.restart_warm_point
+            self._load_suite_params_2, self.options.checkpoint)
+        if self.cli_start_point_string:
+            self.start_point = self.cli_start_point_string
         self.suite_db_mgr.pri_dao.select_broadcast_states(
             self.task_events_mgr.broadcast_mgr.load_db_broadcast_states,
             self.options.checkpoint)
@@ -464,10 +464,10 @@ conditions; see `cylc conditions`.
             if auths:
                 sleep(1.0)
                 # Remote init is done via process pool
-                self.proc_pool.handle_results_async()
+                self.proc_pool.process()
         self.command_poll_tasks()
 
-    def _load_suite_params(self, row_idx, row):
+    def _load_suite_params_2(self, row_idx, row):
         """Load previous initial/final cycle point."""
         if row_idx == 0:
             LOG.info("LOADING suite parameters")
@@ -479,7 +479,7 @@ conditions; see `cylc conditions`.
         for key_str, self_attr, option_ignore_attr in [
                 ("initial", "start_point", "ignore_start_point"),
                 ("final", "stop_point", "ignore_stop_point")]:
-            if key != key_str + "_point" or value is None:
+            if key != key_str + "_point" or value is None or value == 'None':
                 continue
             # the suite_params table prescribes a start/stop cycle
             # (else we take whatever the suite.rc file gives us)
@@ -523,22 +523,42 @@ conditions; see `cylc conditions`.
 
     def process_queued_task_messages(self):
         """Handle incoming task messages for each task proxy."""
-        task_id_messages = {}
+        messages = {}
         while self.message_queue.qsize():
             try:
-                task_id, severity, message = self.message_queue.get(
-                    block=False)
+                task_job, event_time, severity, message = (
+                    self.message_queue.get(block=False))
             except Empty:
                 break
             self.message_queue.task_done()
-            task_id_messages.setdefault(task_id, [])
-            task_id_messages[task_id].append((severity, message))
+            if '/' in task_job:  # cycle/task-name/submit-num
+                cycle, task_name, submit_num = task_job.split('/', 2)
+                task_id = TaskID.get(task_name, cycle)
+                submit_num = int(submit_num, 10)
+            else:  # back compat: task-name.cycle
+                task_id = task_job
+                submit_num = None
+            messages.setdefault(task_id, [])
+            messages[task_id].append(
+                (submit_num, event_time, severity, message))
+        # Note on to_poll_tasks: If an incoming message is going to cause a
+        # reverse change to task state, it is desirable to confirm this by
+        # polling.
+        to_poll_tasks = []
         for itask in self.pool.get_tasks():
-            if itask.identity in task_id_messages:
-                for severity, message in task_id_messages[itask.identity]:
-                    self.task_events_mgr.process_message(
-                        itask, severity, message,
-                        self.task_job_mgr.poll_task_jobs, is_incoming=True)
+            message_items = messages.get(itask.identity)
+            if message_items is None:
+                continue
+            should_poll = False
+            for submit_num, event_time, severity, message in message_items:
+                if self.task_events_mgr.process_message(
+                        itask, severity, message, event_time,
+                        self.task_events_mgr.INCOMING_FLAG, submit_num):
+                    should_poll = True
+            if should_poll:
+                to_poll_tasks.append(itask)
+        self.task_job_mgr.poll_task_jobs(
+            self.suite, to_poll_tasks, poll_succ=True)
 
     def process_command_queue(self):
         """Process queued commands."""
@@ -619,8 +639,8 @@ conditions; see `cylc conditions`.
     def info_get_task_jobfile_path(self, task_id):
         """Return task job file path."""
         name, point = TaskID.split(task_id)
-        return self.task_events_mgr.get_task_job_log(
-            self.suite, point, name, tail=self.task_job_mgr.JOB_FILE_BASE)
+        return get_task_job_log(
+            self.suite, point, name, suffix=JOB_LOG_JOB)
 
     def info_get_suite_info(self):
         """Return a dict containing the suite title and description."""
@@ -692,7 +712,7 @@ conditions; see `cylc conditions`.
             ret['descendants'] = self.config.get_first_parent_descendants()
         if full_mode or ERR.update_time and prev_time < ERR.update_time:
             ret['err_content'], ret['err_size'] = self.suite_log.get_lines(
-                self.suite_log.ERR, client_info.get('prev_err_size'))
+                SUITE_ERR, client_info.get('prev_err_size'))
             client_info['prev_err_size'] = ret['err_size']
         client_info['prev_time'] = client_info['time']
         if self.main_loop_intervals:
@@ -754,7 +774,7 @@ conditions; see `cylc conditions`.
 
     def _set_stop(self, stop_mode=None):
         """Set shutdown mode."""
-        self.proc_pool.stop_job_submission()
+        self.proc_pool.set_stopping()
         if stop_mode is None:
             stop_mode = TaskPool.STOP_REQUEST_CLEAN
         self.stop_mode = stop_mode
@@ -881,12 +901,7 @@ conditions; see `cylc conditions`.
         self.configure_suite_environment()
         if self.options.genref or self.options.reftest:
             self.configure_reftest(recon=True)
-        self.suite_db_mgr.put_suite_params(
-            self.run_mode,
-            self.initial_point,
-            self.final_point,
-            self.pool.is_held,
-            self.config.cfg['cylc']['cycle point format'])
+        self.suite_db_mgr.put_suite_params(self)
         cylc.flags.iflag = True
 
     def set_suite_timer(self):
@@ -912,10 +927,8 @@ conditions; see `cylc conditions`.
                     self._get_events_conf(self.EVENT_INACTIVITY_TIMEOUT)),
                 get_current_time_string()))
 
-    def configure_comms_daemon(self):
-        """Create and configure daemon."""
-        self.httpserver = HTTPServer(self.suite)
-        self.port = self.httpserver.get_port()
+    def configure_contact(self):
+        """Create contact file."""
         # Make sure another suite of the same name has not started while this
         # one is starting
         self.suite_srv_files_mgr.detect_old_contact_file(self.suite)
@@ -939,22 +952,25 @@ conditions; see `cylc conditions`.
         mgr = self.suite_srv_files_mgr
         contact_data = {
             mgr.KEY_API: str(self.httpserver.API),
+            mgr.KEY_COMMS_PROTOCOL: glbl_cfg().get(
+                ['communication', 'method']),
             mgr.KEY_DIR_ON_SUITE_HOST: os.environ['CYLC_DIR'],
-            mgr.KEY_NAME: self.suite,
             mgr.KEY_HOST: self.host,
-            mgr.KEY_PROCESS: process_str,
-            mgr.KEY_PORT: str(self.port),
+            mgr.KEY_NAME: self.suite,
             mgr.KEY_OWNER: self.owner,
+            mgr.KEY_PORT: str(self.httpserver.port),
+            mgr.KEY_PROCESS: process_str,
+            mgr.KEY_SSH_USE_LOGIN_SHELL: str(glbl_cfg().get_host_item(
+                'use login shell')),
             mgr.KEY_SUITE_RUN_DIR_ON_SUITE_HOST: self.suite_run_dir,
-            mgr.KEY_TASK_MSG_MAX_TRIES: str(GLOBAL_CFG.get(
+            mgr.KEY_TASK_MSG_MAX_TRIES: str(glbl_cfg().get(
                 ['task messaging', 'maximum number of tries'])),
-            mgr.KEY_TASK_MSG_RETRY_INTVL: str(float(GLOBAL_CFG.get(
+            mgr.KEY_TASK_MSG_RETRY_INTVL: str(float(glbl_cfg().get(
                 ['task messaging', 'retry interval']))),
-            mgr.KEY_TASK_MSG_TIMEOUT: str(float(GLOBAL_CFG.get(
+            mgr.KEY_TASK_MSG_TIMEOUT: str(float(glbl_cfg().get(
                 ['task messaging', 'connection timeout']))),
-            mgr.KEY_VERSION: CYLC_VERSION,
-            mgr.KEY_COMMS_PROTOCOL: GLOBAL_CFG.get(
-                ['communication', 'method'])}
+            mgr.KEY_UUID: self.task_job_mgr.task_remote_mgr.uuid_str,
+            mgr.KEY_VERSION: CYLC_VERSION}
         try:
             mgr.dump_contact_file(self.suite, contact_data)
         except IOError as exc:
@@ -969,8 +985,8 @@ conditions; see `cylc conditions`.
         self.config = SuiteConfig(
             self.suite, self.suiterc, self.template_vars,
             run_mode=self.run_mode,
-            cli_initial_point_string=self._cli_initial_point_string,
-            cli_start_point_string=self._cli_start_point_string,
+            cli_initial_point_string=self.cli_initial_point_string,
+            cli_start_point_string=self.cli_start_point_string,
             cli_final_point_string=self.options.final_point_string,
             is_reload=is_reload,
             mem_log_func=self.profiler.log_memory,
@@ -980,7 +996,7 @@ conditions; see `cylc conditions`.
         )
         self.suiterc_update_time = time()
         # Dump the loaded suiterc for future reference.
-        cfg_logdir = GLOBAL_CFG.get_derived_host_item(
+        cfg_logdir = glbl_cfg().get_derived_host_item(
             self.suite, 'suite config log directory')
         time_str = get_current_time_string(
             override_use_utc=True, use_basic_format=True,
@@ -1019,23 +1035,22 @@ conditions; see `cylc conditions`.
         if self.run_mode != self.config.run_mode:
             self.run_mode = self.config.run_mode
 
-    def _load_initial_cycle_point(self, _, row):
-        """Load previous initial cycle point.
+    def _load_suite_params_1(self, _, row):
+        """Load previous initial cycle point or (warm) start cycle point.
 
-        For restart, it may be missing from "suite.rc", but was specified as a
-        command line argument on cold/warm start.
+        For restart, these may be missing from "suite.rc", but was specified as
+        a command line argument on cold/warm start.
         """
         key, value = row
-        if key == "initial_point":
-            self._cli_initial_point_string = value
+        if key == 'initial_point':
+            self.cli_initial_point_string = value
             self.task_events_mgr.pflag = True
-
-    def _load_warm_cycle_point(self, _, row):
-        """Load previous warm start point on restart"""
-        key, value = row
-        if key == "warm_point":
-            self._cli_start_point_string = value
-            self.restart_warm_point = value
+        elif key in ['start_point', 'warm_point']:
+            # 'warm_point' for back compat <= 7.6.X
+            self.cli_start_point_string = value
+            self.task_events_mgr.pflag = True
+        elif key == 'uuid_str':
+            self.task_job_mgr.task_remote_mgr.uuid_str = str(value)
 
     def _load_template_vars(self, _, row):
         """Load suite start up template variables."""
@@ -1067,9 +1082,9 @@ conditions; see `cylc conditions`.
         for var, val in [
                 ('CYLC_SUITE_RUN_DIR', self.suite_run_dir),
                 ('CYLC_SUITE_LOG_DIR', self.suite_log.get_dir()),
-                ('CYLC_SUITE_WORK_DIR', GLOBAL_CFG.get_derived_host_item(
+                ('CYLC_SUITE_WORK_DIR', glbl_cfg().get_derived_host_item(
                     self.suite, 'suite work directory')),
-                ('CYLC_SUITE_SHARE_DIR', GLOBAL_CFG.get_derived_host_item(
+                ('CYLC_SUITE_SHARE_DIR', glbl_cfg().get_derived_host_item(
                     self.suite, 'suite share directory')),
                 ('CYLC_SUITE_DEF_PATH', self.suite_dir)]:
             os.environ[var] = val
@@ -1139,7 +1154,8 @@ conditions; see `cylc conditions`.
             pass
         try:
             self.suite_event_handler.handle(self.config, SuiteEventContext(
-                event, reason, self.suite, self.owner, self.host, self.port))
+                event, reason, self.suite, self.owner, self.host,
+                self.httpserver.port))
         except SuiteEventError as exc:
             if event == self.EVENT_SHUTDOWN and self.options.reftest:
                 LOG.error('SUITE REFERENCE TEST FAILED')
@@ -1222,6 +1238,22 @@ conditions; see `cylc conditions`.
             # Something has to be very wrong here, so stop the suite
             raise SchedulerError(str(exc))
 
+    def late_tasks_check(self):
+        """Report tasks that are never active and are late."""
+        now = time()
+        for itask in self.pool.get_tasks():
+            if (not itask.is_late and itask.get_late_time() and
+                    itask.state.status in TASK_STATUSES_NEVER_ACTIVE and
+                    now > itask.get_late_time()):
+                msg = '%s (late-time=%s)' % (
+                    self.task_events_mgr.EVENT_LATE,
+                    time2str(itask.get_late_time()))
+                itask.is_late = True
+                LOG.warning(msg, itask=itask)
+                self.task_events_mgr.setup_event_handlers(
+                    itask, self.task_events_mgr.EVENT_LATE, msg)
+                self.suite_db_mgr.put_insert_task_late_flags(itask)
+
     def timeout_check(self):
         """Check suite and task timers."""
         self.check_suite_timer()
@@ -1266,12 +1298,12 @@ conditions; see `cylc conditions`.
                 stop_process_pool_empty_msg = (
                     "Waiting for the command process pool to empty" +
                     " for shutdown")
-                while not self.proc_pool.is_dead():
+                while self.proc_pool.is_not_done():
                     sleep(self.INTERVAL_STOP_PROCESS_POOL_EMPTY)
                     if stop_process_pool_empty_msg:
                         LOG.info(stop_process_pool_empty_msg)
                         stop_process_pool_empty_msg = None
-                    self.proc_pool.handle_results_async()
+                    self.proc_pool.process()
                     self.process_command_queue()
             if self.options.profile_mode:
                 self.profiler.log_memory(
@@ -1346,12 +1378,13 @@ conditions; see `cylc conditions`.
             if self.pool.release_runahead_tasks():
                 cylc.flags.iflag = True
                 self.task_events_mgr.pflag = True
-            self.proc_pool.handle_results_async()
+            self.proc_pool.process()
 
             # PROCESS ALL TASKS whenever something has changed that might
             # require renegotiation of dependencies, etc.
-            if self.process_tasks():
+            if self.should_process_tasks():
                 self.process_task_pool()
+            self.late_tasks_check()
 
             self.process_queued_task_messages()
             self.process_command_queue()
@@ -1381,7 +1414,21 @@ conditions; see `cylc conditions`.
             if self.options.profile_mode:
                 self.update_profiler_logs(tinit)
 
-            sleep(self.INTERVAL_MAIN_LOOP)
+            # Sleep a bit for things to catch up.
+            # Quick sleep if there are items pending in process pool.
+            # (Should probably use quick sleep logic for other queues?)
+            elapsed = time() - tinit
+            quick_mode = self.proc_pool.is_not_done()
+            if (elapsed >= self.INTERVAL_MAIN_LOOP or
+                    quick_mode and elapsed >= self.INTERVAL_MAIN_LOOP_QUICK):
+                # Main loop has taken quite a bit to get through
+                # Still yield control to other threads by sleep(0.0)
+                sleep(0.0)
+            elif quick_mode:
+                sleep(self.INTERVAL_MAIN_LOOP_QUICK - elapsed)
+            else:
+                sleep(self.INTERVAL_MAIN_LOOP - elapsed)
+            # Record latest main loop interval
             self.main_loop_intervals.append(time() - tinit)
             # END MAIN LOOP
 
@@ -1444,22 +1491,13 @@ conditions; see `cylc conditions`.
             if self._get_events_conf(self.EVENT_TIMEOUT):
                 self.set_suite_timer()
 
-    def process_tasks(self):
+    def should_process_tasks(self):
         """Return True if waiting tasks are ready."""
         # do we need to do a pass through the main task processing loop?
         process = False
 
-        # External triggers must be matched now. If any are matched pflag
-        # is set to tell process_tasks() that task processing is required.
-        broadcast_mgr = self.task_events_mgr.broadcast_mgr
-        broadcast_mgr.add_ext_triggers(self.ext_trigger_queue)
-        for itask in self.pool.get_tasks():
-            if (itask.state.external_triggers and
-                    broadcast_mgr.match_ext_trigger(itask)):
-                process = True
-
         if self.task_events_mgr.pflag:
-            # this flag is turned on by commands that change task state
+            # This flag is turned on by commands that change task state
             process = True
             self.task_events_mgr.pflag = False  # reset
 
@@ -1468,10 +1506,18 @@ conditions; see `cylc conditions`.
             process = True
             self.task_job_mgr.task_remote_mgr.ready = False  # reset
 
-        self.pool.set_expired_tasks()
-        if self.pool.waiting_tasks_ready():
-            process = True
-
+        broadcast_mgr = self.task_events_mgr.broadcast_mgr
+        broadcast_mgr.add_ext_triggers(self.ext_trigger_queue)
+        now = time()
+        for itask in self.pool.get_tasks():
+            # External trigger matching and task expiry must be done
+            # regardless, so they need to be in separate "if ..." blocks.
+            if broadcast_mgr.match_ext_trigger(itask):
+                process = True
+            if self.pool.set_expired_task(itask, now):
+                process = True
+            if itask.is_ready(now):
+                process = True
         if self.run_mode == 'simulation' and self.pool.sim_time_check(
                 self.message_queue):
             process = True
@@ -1495,7 +1541,8 @@ conditions; see `cylc conditions`.
             try:
                 handle = open(
                     os.path.join(self.config.fdir, 'reference.log'), 'wb')
-                for line in open(self.suite_log.get_log_path(SuiteLog.LOG)):
+                for line in open(
+                        self.suite_log.get_log_path(SUITE_LOG)):
                     if any(text in line for text in self.REF_LOG_TEXTS):
                         handle.write(line)
                 handle.close()
@@ -1503,11 +1550,10 @@ conditions; see `cylc conditions`.
                 ERR.error(str(exc))
 
         if self.proc_pool:
-            if not self.proc_pool.is_dead():
+            if self.proc_pool.is_not_done():
                 # e.g. KeyboardInterrupt
                 self.proc_pool.terminate()
-            self.proc_pool.join()
-            self.proc_pool.handle_results_async()
+            self.proc_pool.process()
 
         if self.pool is not None:
             self.pool.warn_stop_orphans()
@@ -1625,8 +1671,8 @@ conditions; see `cylc conditions`.
             elif itasks[0] in bad_tasks:
                 return n_warnings + 1
             else:
-                self.task_job_mgr.proc_pool.handle_results_async()
-                sleep(1.0)
+                self.proc_pool.process()
+                sleep(self.INTERVAL_MAIN_LOOP_QUICK)
 
     def command_reset_task_states(self, items, state=None, outputs=None):
         """Reset the state of tasks."""
@@ -1711,7 +1757,7 @@ conditions; see `cylc conditions`.
 
     def _get_cylc_conf(self, key, default=None):
         """Return a named setting under [cylc] from suite.rc or global.rc."""
-        for getter in [self.config.cfg['cylc'], GLOBAL_CFG.get(['cylc'])]:
+        for getter in [self.config.cfg['cylc'], glbl_cfg().get(['cylc'])]:
             try:
                 value = getter[key]
             except KeyError:
