@@ -1,4 +1,4 @@
-#!/usr/bin/env python
+#!/usr/bin/env python2
 
 # THIS FILE IS PART OF THE CYLC SUITE ENGINE.
 # Copyright (C) 2008-2018 NIWA
@@ -21,9 +21,11 @@
 # imported on demand.
 import os
 import re
-from string import ascii_letters, digits
 import sys
+from uuid import uuid4
+from string import ascii_letters, digits
 
+from cylc.cfgspec.glbl_cfg import glbl_cfg
 import cylc.flags
 from cylc.mkdir_p import mkdir_p
 from cylc.hostuserutil import (
@@ -42,6 +44,7 @@ class SuiteSrvFilesManager(object):
     DIR_BASE_AUTH = "auth"
     DIR_BASE_SRV = ".service"
     FILE_BASE_CONTACT = "contact"
+    FILE_BASE_CONTACT2 = "contact2"
     FILE_BASE_PASSPHRASE = "passphrase"
     FILE_BASE_SOURCE = "source"
     FILE_BASE_SSL_CERT = "ssl.cert"
@@ -49,16 +52,19 @@ class SuiteSrvFilesManager(object):
     FILE_BASE_SUITE_RC = "suite.rc"
     KEY_API = "CYLC_API"
     KEY_COMMS_PROTOCOL = "CYLC_COMMS_PROTOCOL"  # default (or none?)
+    KEY_COMMS_PROTOCOL_2 = "CYLC_COMMS_PROTOCOL_2"  # indirect comms
     KEY_DIR_ON_SUITE_HOST = "CYLC_DIR_ON_SUITE_HOST"
     KEY_HOST = "CYLC_SUITE_HOST"
     KEY_NAME = "CYLC_SUITE_NAME"
     KEY_OWNER = "CYLC_SUITE_OWNER"
     KEY_PROCESS = "CYLC_SUITE_PROCESS"
     KEY_PORT = "CYLC_SUITE_PORT"
+    KEY_SSH_USE_LOGIN_SHELL = "CYLC_SSH_USE_LOGIN_SHELL"
     KEY_SUITE_RUN_DIR_ON_SUITE_HOST = "CYLC_SUITE_RUN_DIR_ON_SUITE_HOST"
     KEY_TASK_MSG_MAX_TRIES = "CYLC_TASK_MSG_MAX_TRIES"
     KEY_TASK_MSG_RETRY_INTVL = "CYLC_TASK_MSG_RETRY_INTVL"
     KEY_TASK_MSG_TIMEOUT = "CYLC_TASK_MSG_TIMEOUT"
+    KEY_UUID = "CYLC_SUITE_UUID"
     KEY_VERSION = "CYLC_VERSION"
     NO_TITLE = "No title provided"
     PASSPHRASE_CHARSET = ascii_letters + digits
@@ -138,8 +144,7 @@ class SuiteSrvFilesManager(object):
         cmd = ["timeout", "10", "ps", self.PS_OPTS, str(old_pid_str)]
         if is_remote_host(old_host):
             import shlex
-            from cylc.cfgspec.globalcfg import GLOBAL_CFG
-            ssh_str = str(GLOBAL_CFG.get_host_item("ssh command", old_host))
+            ssh_str = str(glbl_cfg().get_host_item("ssh command", old_host))
             cmd = shlex.split(ssh_str) + ["-n", old_host] + cmd
         from subprocess import Popen, PIPE
         from time import sleep, time
@@ -233,7 +238,7 @@ To start a new run, stop the old one first with one or more of these:
         if item not in [
                 self.FILE_BASE_SSL_CERT, self.FILE_BASE_SSL_PEM,
                 self.FILE_BASE_PASSPHRASE, self.FILE_BASE_CONTACT,
-                self.KEY_COMMS_PROTOCOL]:
+                self.FILE_BASE_CONTACT2]:
             raise ValueError("%s: item not recognised" % item)
         if item == self.FILE_BASE_PASSPHRASE:
             self.can_disk_cache_passphrases[(reg, owner, host)] = False
@@ -290,15 +295,19 @@ To start a new run, stop the old one first with one or more of these:
                     return value
 
         # 5/ Use SSH to load content from remote owner@host
-        value = self._load_remote_item(item, reg, owner, host)
-        if value:
-            if item == self.FILE_BASE_PASSPHRASE:
-                self.can_disk_cache_passphrases[(reg, owner, host)] = True
-            if not content:
-                path = self._get_cache_dir(reg, owner, host)
-                self._dump_item(path, item, value)
-                value = os.path.join(path, item)
-            return value
+        # Note: It is not possible to find ".service/contact2" on the suite
+        # host, because it is installed on task host by "cylc remote-init" on
+        # demand.
+        if item != self.FILE_BASE_CONTACT2:
+            value = self._load_remote_item(item, reg, owner, host)
+            if value:
+                if item == self.FILE_BASE_PASSPHRASE:
+                    self.can_disk_cache_passphrases[(reg, owner, host)] = True
+                if not content:
+                    path = self._get_cache_dir(reg, owner, host)
+                    self._dump_item(path, item, value)
+                    value = os.path.join(path, item)
+                return value
 
         raise SuiteServiceFileError("Couldn't get %s" % item)
 
@@ -329,8 +338,7 @@ To start a new run, stop the old one first with one or more of these:
         run_d = os.getenv("CYLC_SUITE_RUN_DIR")
         if (not run_d or os.getenv("CYLC_SUITE_NAME") != reg or
                 os.getenv("CYLC_SUITE_OWNER") != suite_owner):
-            from cylc.cfgspec.globalcfg import GLOBAL_CFG
-            run_d = GLOBAL_CFG.get_derived_host_item(
+            run_d = glbl_cfg().get_derived_host_item(
                 reg, 'suite run directory')
         return os.path.join(run_d, self.DIR_BASE_SRV)
 
@@ -342,8 +350,7 @@ To start a new run, stop the old one first with one or more of these:
                 rec_regfilter = re.compile(regfilter)
             except re.error as exc:
                 raise ValueError("%s: %s" % (regfilter, exc))
-        from cylc.cfgspec.globalcfg import GLOBAL_CFG
-        run_d = GLOBAL_CFG.get_host_item('run directory')
+        run_d = glbl_cfg().get_host_item('run directory')
         results = []
         for dirpath, dnames, fnames in os.walk(run_d, followlinks=True):
             # Always descend for top directory, but
@@ -368,10 +375,12 @@ To start a new run, stop the old one first with one or more of these:
                 print >> sys.stderr, str(exc)
         return results
 
-    def load_contact_file(self, reg, owner=None, host=None):
+    def load_contact_file(self, reg, owner=None, host=None, file_base=None):
         """Load contact file. Return data as key=value dict."""
+        if not file_base:
+            file_base = self.FILE_BASE_CONTACT
         file_content = self.get_auth_item(
-            self.FILE_BASE_CONTACT, reg, owner, host, content=True)
+            file_base, reg, owner, host, content=True)
         data = {}
         for line in file_content.splitlines():
             key, value = [item.strip() for item in line.split("=", 1)]
@@ -387,6 +396,8 @@ To start a new run, stop the old one first with one or more of these:
         If arg is a file, suite name is the base name of its container
         directory.
         """
+        if arg == '.':
+            arg = os.getcwd()
         try:
             path = self.get_suite_rc(arg, options.suite_owner)
             name = arg
@@ -516,7 +527,9 @@ To start a new run, stop the old one first with one or more of these:
         cert_obj.gmtime_adj_notAfter(10 * 365 * 24 * 60 * 60)  # 10 years.
         cert_obj.set_issuer(cert_obj.get_subject())
         cert_obj.set_pubkey(pkey_obj)
-        cert_obj.set_serial_number(1)
+        # Set a random serial number to avoid browser error
+        # SEC_ERROR_REUSED_ISSUER_AND_SERIAL.
+        cert_obj.set_serial_number(uuid4().int)
         cert_obj.add_extensions([ext])
         cert_obj.sign(pkey_obj, 'sha256')
         self._dump_item(
@@ -623,11 +636,10 @@ To start a new run, stop the old one first with one or more of these:
             host = 'localhost'
         if owner is None:
             owner = get_user()
-        from cylc.cfgspec.globalcfg import GLOBAL_CFG
-        if item == 'contact' and not is_remote_host(host):
+        if item == self.FILE_BASE_CONTACT and not is_remote_host(host):
             # Attempt to read suite contact file via the local filesystem.
             path = r'%(run_d)s/%(srv_base)s' % {
-                'run_d': GLOBAL_CFG.get_derived_host_item(
+                'run_d': glbl_cfg().get_derived_host_item(
                     reg, 'suite run directory', 'localhost', owner,
                     replace_home=False),
                 'srv_base': self.DIR_BASE_SRV,
@@ -644,14 +656,14 @@ To start a new run, stop the old one first with one or more of these:
             r'''cat "%(run_d)s/%(srv_base)s/%(item)s"'''
         ) % {
             'prefix': prefix,
-            'run_d': GLOBAL_CFG.get_derived_host_item(
+            'run_d': glbl_cfg().get_derived_host_item(
                 reg, 'suite run directory', host, owner),
             'srv_base': self.DIR_BASE_SRV,
             'item': item
         }
         import shlex
         command = shlex.split(
-            GLOBAL_CFG.get_host_item('ssh command', host, owner))
+            glbl_cfg().get_host_item('ssh command', host, owner))
         command += ['-n', owner + '@' + host, script]
         from subprocess import Popen, PIPE
         try:
