@@ -1,4 +1,4 @@
-#!/usr/bin/env python
+#!/usr/bin/env python2
 
 # THIS FILE IS PART OF THE CYLC SUITE ENGINE.
 # Copyright (C) 2008-2018 NIWA
@@ -31,7 +31,6 @@ from collections import namedtuple
 from logging import getLevelName, CRITICAL, ERROR, WARNING, INFO, DEBUG
 import os
 from pipes import quote
-import re
 import shlex
 from time import time
 import traceback
@@ -39,22 +38,26 @@ import traceback
 from parsec.config import ItemNotFoundError
 
 from cylc.broadcast_mgr import BroadcastMgr
-from cylc.cfgspec.globalcfg import GLOBAL_CFG
+from cylc.cfgspec.glbl_cfg import glbl_cfg
 import cylc.flags
 from cylc.mp_pool import SuiteProcContext
 from cylc.suite_logging import ERR, LOG
 from cylc.hostuserutil import get_host, get_user
 from cylc.task_action_timer import TaskActionTimer
-from cylc.task_message import TaskMessage
+from cylc.task_job_logs import (
+    get_task_job_log, get_task_job_activity_log, JOB_LOG_OUT, JOB_LOG_ERR)
+from cylc.task_message import (
+    ABORT_MESSAGE_PREFIX, FAIL_MESSAGE_PREFIX, VACATION_MESSAGE_PREFIX)
 from cylc.task_state import (
+    TASK_STATUSES_ACTIVE,
     TASK_STATUS_READY, TASK_STATUS_SUBMITTED, TASK_STATUS_SUBMIT_RETRYING,
     TASK_STATUS_SUBMIT_FAILED, TASK_STATUS_RUNNING, TASK_STATUS_RETRYING,
     TASK_STATUS_FAILED, TASK_STATUS_SUCCEEDED)
 from cylc.task_outputs import (
     TASK_OUTPUT_SUBMITTED, TASK_OUTPUT_STARTED, TASK_OUTPUT_SUCCEEDED,
-    TASK_OUTPUT_FAILED)
+    TASK_OUTPUT_FAILED, TASK_OUTPUT_SUBMIT_FAILED, TASK_OUTPUT_EXPIRED)
 from cylc.wallclock import (
-    get_current_time_string, RE_DATE_TIME_FORMAT_EXTENDED)
+    get_current_time_string, get_seconds_as_interval_string as intvl_as_str)
 
 
 CustomTaskEventHandlerContext = namedtuple(
@@ -72,6 +75,30 @@ TaskJobLogsRetrieveContext = namedtuple(
     ["key", "ctx_type", "user_at_host", "max_size"])
 
 
+def log_task_job_activity(ctx, suite, point, name, submit_num=None):
+    """Log an activity for a task job."""
+    ctx_str = str(ctx)
+    if not ctx_str:
+        return
+    if isinstance(ctx.cmd_key, tuple):  # An event handler
+        submit_num = ctx.cmd_key[-1]
+    job_activity_log = get_task_job_activity_log(
+        suite, point, name, submit_num)
+    try:
+        with open(job_activity_log, "ab") as handle:
+            handle.write(ctx_str + '\n')
+    except IOError as exc:
+        # This happens when there is no job directory, e.g. if job host
+        # selection command causes an submission failure, there will be no job
+        # directory. In this case, just send the information to the suite log.
+        LOG.debug(exc)
+        LOG.info(ctx_str)
+    if ctx.cmd and ctx.ret_code:
+        LOG.error(ctx_str)
+    elif ctx.cmd:
+        LOG.debug(ctx_str)
+
+
 class TaskEventsManager(object):
     """Task events manager.
 
@@ -81,6 +108,7 @@ class TaskEventsManager(object):
     * Generate and manage task event handlers.
     """
     EVENT_FAILED = TASK_OUTPUT_FAILED
+    EVENT_LATE = "late"
     EVENT_RETRY = "retry"
     EVENT_STARTED = TASK_OUTPUT_STARTED
     EVENT_SUBMITTED = TASK_OUTPUT_SUBMITTED
@@ -91,7 +119,9 @@ class TaskEventsManager(object):
     HANDLER_MAIL = "event-mail"
     JOB_FAILED = "job failed"
     HANDLER_JOB_LOGS_RETRIEVE = "job-logs-retrieve"
+    IGNORED_INCOMING_FLAG = "(>ignored)"
     INCOMING_FLAG = ">"
+    KEY_EXECUTE_TIME_LIMIT = 'execution_time_limit'
     LEVELS = {
         "INFO": INFO,
         "NORMAL": INFO,
@@ -100,10 +130,7 @@ class TaskEventsManager(object):
         "CRITICAL": CRITICAL,
         "DEBUG": DEBUG,
     }
-    NN = "NN"
-    POLLED_INDICATOR = "(polled)"
-    RE_MESSAGE_TIME = re.compile(
-        '\A(.+) at (' + RE_DATE_TIME_FORMAT_EXTENDED + ')\Z', re.DOTALL)
+    POLLED_FLAG = "(polled)"
 
     def __init__(self, suite, proc_pool, suite_db_mgr, broadcast_mgr=None):
         self.suite = suite
@@ -125,6 +152,57 @@ class TaskEventsManager(object):
         # when required.
         self.pflag = False
 
+    @staticmethod
+    def check_poll_time(itask, now=None):
+        """Set the next task execution/submission poll time.
+
+        If now is set, set the timer only if the previous delay is done.
+        Return the next delay.
+        """
+        if itask.state.status not in TASK_STATUSES_ACTIVE:
+            # Reset, task not active
+            itask.timeout = None
+            itask.poll_timer = None
+            return None
+        ctx = (itask.submit_num, itask.state.status)
+        if itask.poll_timer is None or itask.poll_timer.ctx != ctx:
+            # Reset, timer no longer relevant
+            itask.timeout = None
+            itask.poll_timer = None
+            return None
+        if now is not None and not itask.poll_timer.is_delay_done(now):
+            return False
+        if itask.poll_timer.num is None:
+            itask.poll_timer.num = 0
+        itask.poll_timer.next(no_exhaust=True)
+        return True
+
+    def check_job_time(self, itask, now):
+        """Check/handle job timeout and poll timer"""
+        can_poll = self.check_poll_time(itask, now)
+        if itask.timeout is None or now <= itask.timeout:
+            return can_poll
+        # Timeout reached for task, emit event and reset itask.timeout
+        if itask.state.status == TASK_STATUS_RUNNING:
+            time_ref = itask.summary['started_time']
+            event = 'execution timeout'
+        elif itask.state.status == TASK_STATUS_SUBMITTED:
+            time_ref = itask.summary['submitted_time']
+            event = 'submission timeout'
+        msg = event
+        try:
+            msg += ' after %s' % intvl_as_str(itask.timeout - time_ref)
+        except (TypeError, ValueError):
+            # Badness in time_ref?
+            pass
+        itask.timeout = None  # emit event only once
+        if msg and event:
+            LOG.warning(msg, itask=itask)
+            self.setup_event_handlers(itask, event, msg)
+            return True
+        else:
+            return can_poll
+
     def get_host_conf(self, itask, key, default=None, skey="remote"):
         """Return a host setting from suite then global configuration."""
         overrides = self.broadcast_mgr.get_broadcast(itask.identity)
@@ -134,55 +212,12 @@ class TaskEventsManager(object):
             return itask.tdef.rtconfig[skey][key]
         else:
             try:
-                return GLOBAL_CFG.get_host_item(
+                return glbl_cfg().get_host_item(
                     key, itask.task_host, itask.task_owner)
             except (KeyError, ItemNotFoundError):
                 pass
         return default
 
-    def get_task_job_activity_log(
-            self, suite, point, name, submit_num=None):
-        """Shorthand for get_task_job_log(..., tail="job-activity.log")."""
-        return self.get_task_job_log(
-            suite, point, name, submit_num, "job-activity.log")
-
-    def get_task_job_log(
-            self, suite, point, name, submit_num=None, tail=None):
-        """Return the job log path."""
-        args = [
-            GLOBAL_CFG.get_derived_host_item(suite, "suite job log directory"),
-            self.get_task_job_id(point, name, submit_num)]
-        if tail:
-            args.append(tail)
-        return os.path.join(*args)
-
-    def get_task_job_id(self, point, name, submit_num=None):
-        """Return the job log path."""
-        try:
-            submit_num = "%02d" % submit_num
-        except TypeError:
-            submit_num = self.NN
-        return os.path.join(str(point), name, submit_num)
-
-    def log_task_job_activity(self, ctx, suite, point, name, submit_num=NN):
-        """Log an activity for a task job."""
-        ctx_str = str(ctx)
-        if not ctx_str:
-            return
-        if isinstance(ctx.cmd_key, tuple):  # An event handler
-            submit_num = ctx.cmd_key[-1]
-        job_activity_log = self.get_task_job_activity_log(
-            suite, point, name, submit_num)
-        try:
-            with open(job_activity_log, "ab") as handle:
-                handle.write(ctx_str + '\n')
-        except IOError as exc:
-            LOG.warning("%s: write failed\n%s" % (job_activity_log, exc))
-        if ctx.cmd and ctx.ret_code:
-            LOG.error(ctx_str)
-        elif ctx.cmd:
-            LOG.debug(ctx_str)
-
     def process_events(self, schd_ctx):
         """Process task events that were created by "setup_event_handlers".
 
@@ -204,14 +239,13 @@ class TaskEventsManager(object):
                 # Report retries and delayed 1st try
                 tmpl = None
                 if timer.num > 1:
-                    tmpl = "%s/%s/%02d %s failed, retrying in %s (after %s)"
+                    tmpl = "%s/%s/%02d %s failed, retrying in %s"
                 elif timer.delay:
-                    tmpl = "%s/%s/%02d %s will run after %s (after %s)"
+                    tmpl = "%s/%s/%02d %s will run after %s"
                 if tmpl:
                     LOG.debug(tmpl % (
                         point, name, submit_num, key1,
-                        timer.delay_as_seconds(),
-                        timer.timeout_as_str()))
+                        timer.delay_timeout_as_str()))
             # Ready to run?
             if not timer.is_delay_done() or (
                 # Avoid flooding user's mail box with mail notification.
@@ -248,20 +282,9 @@ class TaskEventsManager(object):
             elif ctx.ctx_type == self.HANDLER_JOB_LOGS_RETRIEVE:
                 self._process_job_logs_retrieval(schd_ctx, ctx, id_keys)
 
-    def _poll_to_confirm(self, itask, status_gt, poll_func):
-        """Poll itask to confirm an apparent state reversal."""
-        if (itask.state.is_greater_than(status_gt) and not
-                itask.state.confirming_with_poll):
-            poll_func(self.suite, [itask],
-                      msg="polling %s to confirm state" % itask.identity)
-            itask.state.confirming_with_poll = True
-            return True
-        else:
-            itask.state.confirming_with_poll = False
-            return False
-
-    def process_message(self, itask, severity, message, poll_func,
-                        poll_event_time=None, is_incoming=False):
+    def process_message(
+            self, itask, severity, message, event_time=None, flag='',
+            submit_num=None):
         """Parse an incoming task message and update task state.
 
         Incoming, e.g. "succeeded at <TIME>", may be from task job or polling.
@@ -276,32 +299,51 @@ class TaskEventsManager(object):
         but this is the best we can do without somehow uniquely associating
         each poll with its result message.
 
+        Arguments:
+            itask (cylc.task_proxy.TaskProxy):
+                The task proxy object relevant for the message.
+            severity (str or int):
+                Message severity, should be a recognised logging level.
+            message (str):
+                Message content.
+            event_time (str):
+                Event time stamp. Expect ISO8601 date time string.
+                If not specified, use current time.
+            flag (str):
+                If specified, can be INCOMING_FLAG to indicate an incoming
+                message, POLLED_FLAG to indicate a message resulted from a
+                poll. Otherwise, the message is assumed to be generated by the
+                logic in the suite server program.
+            submit_num (int):
+                The submit number of the task relevant for the message.
+                If not specified, use latest submit number.
+
+        Return:
+            None: in normal circumstances.
+            True: if polling is required to confirm a reversal of status.
+
         """
-
-        is_polled = poll_event_time is not None
-        # Log incoming messages with '>' to distinguish non-message log entries
-        message_flag = ""
-        if is_incoming:
-            message_flag = self.INCOMING_FLAG
-        log_message = '(current:%s)%s %s' % (
-            itask.state.status, message_flag, message)
-        if poll_event_time is not None:
-            log_message += ' %s' % self.POLLED_INDICATOR
-        LOG.log(self.LEVELS.get(severity, INFO), log_message, itask=itask)
-
-        # Strip the "at TIME" suffix.
-        event_time = poll_event_time
-        if not event_time:
-            match = self.RE_MESSAGE_TIME.match(message)
-            if match:
-                message, event_time = match.groups()
-        if not event_time:
+        # Log incoming messages
+        if event_time is None:
             event_time = get_current_time_string()
+        if submit_num is None:
+            submit_num = itask.submit_num
+        if flag == self.INCOMING_FLAG and submit_num != itask.submit_num:
+            flag = self.IGNORED_INCOMING_FLAG
+        log_message = '(current:%s)%s %s at %s' % (
+            itask.state.status, flag, message, event_time)
+        LOG.log(self.LEVELS.get(severity, INFO), log_message, itask=itask)
+        if flag == self.IGNORED_INCOMING_FLAG:
+            LOG.warning(
+                'submit-num=%d: ignore message from job submit-num=%d' % (
+                    itask.submit_num, submit_num),
+                itask=itask)
+            return
 
         # always update the suite state summary for latest message
         itask.summary['latest_message'] = message
-        if is_polled:
-            itask.summary['latest_message'] += " %s" % self.POLLED_INDICATOR
+        if flag == self.POLLED_FLAG:
+            itask.summary['latest_message'] += ' %s' % self.POLLED_FLAG
         cylc.flags.iflag = True
 
         # Satisfy my output, if possible, and record the result.
@@ -309,60 +351,58 @@ class TaskEventsManager(object):
             message=message, is_completed=True)
 
         if message == TASK_OUTPUT_STARTED:
-            if self._poll_to_confirm(itask, TASK_STATUS_RUNNING, poll_func):
-                return
+            if (flag == self.INCOMING_FLAG
+                    and itask.state.is_gt(TASK_STATUS_RUNNING)):
+                return True
             self._process_message_started(itask, event_time)
         elif message == TASK_OUTPUT_SUCCEEDED:
-            if self._poll_to_confirm(itask, TASK_STATUS_SUCCEEDED, poll_func):
-                return
             self._process_message_succeeded(itask, event_time)
         elif message == TASK_OUTPUT_FAILED:
-            if self._poll_to_confirm(itask, TASK_STATUS_FAILED, poll_func):
-                return
+            if (flag == self.INCOMING_FLAG
+                    and itask.state.is_gt(TASK_STATUS_FAILED)):
+                return True
             self._process_message_failed(itask, event_time, self.JOB_FAILED)
         elif message == self.EVENT_SUBMIT_FAILED:
-            if self._poll_to_confirm(itask,
-                                     TASK_STATUS_SUBMIT_FAILED, poll_func):
-                return
+            if (flag == self.INCOMING_FLAG
+                    and itask.state.is_gt(TASK_STATUS_SUBMIT_FAILED)):
+                return True
             self._process_message_submit_failed(itask, event_time)
         elif message == TASK_OUTPUT_SUBMITTED:
-            if self._poll_to_confirm(itask, TASK_STATUS_SUBMITTED, poll_func):
-                return
+            if (flag == self.INCOMING_FLAG
+                    and itask.state.is_gt(TASK_STATUS_SUBMITTED)):
+                return True
             self._process_message_submitted(itask, event_time)
-        elif message.startswith(TaskMessage.FAIL_MESSAGE_PREFIX):
+        elif message.startswith(FAIL_MESSAGE_PREFIX):
             # Task received signal.
-            signal = message[len(TaskMessage.FAIL_MESSAGE_PREFIX):]
+            if (flag == self.INCOMING_FLAG
+                    and itask.state.is_gt(TASK_STATUS_FAILED)):
+                return True
+            signal = message[len(FAIL_MESSAGE_PREFIX):]
             self._db_events_insert(itask, "signaled", signal)
             self.suite_db_mgr.put_update_task_jobs(
                 itask, {"run_signal": signal})
-            if self._poll_to_confirm(itask, TASK_STATUS_FAILED, poll_func):
-                return
             self._process_message_failed(itask, event_time, self.JOB_FAILED)
-        elif message.startswith(TaskMessage.ABORT_MESSAGE_PREFIX):
+        elif message.startswith(ABORT_MESSAGE_PREFIX):
             # Task aborted with message
-            aborted_with = message[len(TaskMessage.ABORT_MESSAGE_PREFIX):]
+            if (flag == self.INCOMING_FLAG
+                    and itask.state.is_gt(TASK_STATUS_FAILED)):
+                return True
+            aborted_with = message[len(ABORT_MESSAGE_PREFIX):]
             self._db_events_insert(itask, "aborted", message)
             self.suite_db_mgr.put_update_task_jobs(
                 itask, {"run_signal": aborted_with})
-            if self._poll_to_confirm(itask, TASK_STATUS_FAILED, poll_func):
-                return
             self._process_message_failed(itask, event_time, aborted_with)
-        elif message.startswith(TaskMessage.VACATION_MESSAGE_PREFIX):
+        elif message.startswith(VACATION_MESSAGE_PREFIX):
             # Task job pre-empted into a vacation state
             self._db_events_insert(itask, "vacated", message)
-            itask.set_event_time('started')  # reset
+            itask.set_summary_time('started')  # unset
             if TASK_STATUS_SUBMIT_RETRYING in itask.try_timers:
                 itask.try_timers[TASK_STATUS_SUBMIT_RETRYING].num = 0
             itask.job_vacated = True
-            try:
-                itask.timeout_timers[TASK_STATUS_SUBMITTED] = (
-                    itask.summary['submitted_time'] +
-                    float(self._get_events_conf(itask, 'submission timeout')))
-            except (TypeError, ValueError):
-                itask.timeout_timers[TASK_STATUS_SUBMITTED] = None
             # Believe this and change state without polling (could poll?).
             self.pflag = True
             itask.state.reset_state(TASK_STATUS_SUBMITTED)
+            self._reset_job_timers(itask)
         elif an_output_was_satisfied:
             # Message of an as-yet unreported custom task output.
             # No state change.
@@ -375,15 +415,13 @@ class TaskEventsManager(object):
             # Note that all messages are logged already at the top.
             # No state change.
             LOG.debug(
-                '(current: %s) unhandled: %s' % (itask.state.status, message),
+                '(current:%s) unhandled: %s' % (itask.state.status, message),
                 itask=itask)
             if severity in [CRITICAL, ERROR, WARNING, INFO, DEBUG]:
                 severity = getLevelName(severity)
             self._db_events_insert(
                 itask, ("message %s" % str(severity).lower()), message)
-        if severity == "CUSTOM":
-            self.setup_event_handlers(itask, "custom", message)
-        elif severity in [TaskMessage.WARNING, TaskMessage.CRITICAL]:
+        if severity in ['WARNING', 'CRITICAL', 'CUSTOM']:
             self.setup_event_handlers(itask, severity.lower(), message)
 
     def setup_event_handlers(self, itask, event, message):
@@ -398,34 +436,10 @@ class TaskEventsManager(object):
         self._setup_event_mail(itask, event)
         self._setup_custom_event_handlers(itask, event, message)
 
-    @staticmethod
-    def set_poll_time(itask, now=None):
-        """Set the next task execution/submission poll time.
-
-        If now is set, set the timer only if the previous delay is done.
-        Return the next delay.
-        """
-        key = itask.state.status
-        timer = itask.poll_timers.get(key)
-        if timer is None:
-            return
-        if now is not None and not timer.is_delay_done(now):
-            return
-        if timer.num is None:
-            timer.num = 0
-        delay = timer.next(no_exhaust=True)
-        if delay is not None:
-            LOG.info(
-                'next job poll in %s (after %s)' % (
-                    timer.delay_as_seconds(), timer.timeout_as_str()),
-                itask=itask)
-        return delay
-
     def _custom_handler_callback(self, ctx, schd_ctx, id_key):
         """Callback when a custom event handler is done."""
         _, point, name, submit_num = id_key
-        self.log_task_job_activity(
-            ctx, schd_ctx.suite, point, name, submit_num)
+        log_task_job_activity(ctx, schd_ctx.suite, point, name, submit_num)
         if ctx.ret_code == 0:
             del self.event_timers[id_key]
         else:
@@ -500,7 +514,7 @@ class TaskEventsManager(object):
                     del self.event_timers[id_key]
                     log_ctx = SuiteProcContext((key1, submit_num), None)
                     log_ctx.ret_code = 0
-                    self.log_task_job_activity(
+                    log_task_job_activity(
                         log_ctx, schd_ctx.suite, point, name, submit_num)
                 else:
                     self.event_timers[id_key].unset_waiting()
@@ -513,7 +527,7 @@ class TaskEventsManager(object):
         for getter in [
                 self.broadcast_mgr.get_broadcast(itask.identity).get("events"),
                 itask.tdef.rtconfig["events"],
-                GLOBAL_CFG.get()["task events"]]:
+                glbl_cfg().get()["task events"]]:
             try:
                 value = getter.get(key)
             except (AttributeError, ItemNotFoundError, KeyError):
@@ -529,8 +543,8 @@ class TaskEventsManager(object):
             s_user, s_host = ctx.user_at_host.split("@", 1)
         else:
             s_user, s_host = (None, ctx.user_at_host)
-        ssh_str = str(GLOBAL_CFG.get_host_item("ssh command", s_host, s_user))
-        rsync_str = str(GLOBAL_CFG.get_host_item(
+        ssh_str = str(glbl_cfg().get_host_item("ssh command", s_host, s_user))
+        rsync_str = str(glbl_cfg().get_host_item(
             "retrieve job logs command", s_host, s_user))
 
         cmd = shlex.split(rsync_str) + ["--rsh=" + ssh_str]
@@ -549,10 +563,10 @@ class TaskEventsManager(object):
         cmd += ["--include=%s" % (include) for include in sorted(includes)]
         cmd.append("--exclude=/**")  # exclude everything else
         # Remote source
-        cmd.append(ctx.user_at_host + ":" + GLOBAL_CFG.get_derived_host_item(
+        cmd.append(ctx.user_at_host + ":" + glbl_cfg().get_derived_host_item(
             schd_ctx.suite, "suite job log directory", s_host, s_user) + "/")
         # Local target
-        cmd.append(GLOBAL_CFG.get_derived_host_item(
+        cmd.append(glbl_cfg().get_derived_host_item(
             schd_ctx.suite, "suite job log directory") + "/")
         self.proc_pool.put_command(
             SuiteProcContext(ctx, cmd, env=dict(os.environ), id_keys=id_keys),
@@ -564,15 +578,15 @@ class TaskEventsManager(object):
             key1, point, name, submit_num = id_key
             try:
                 # All completed jobs are expected to have a "job.out".
-                fnames = ["job.out"]
+                fnames = [JOB_LOG_OUT]
                 try:
                     if key1[1] not in 'succeeded':
-                        fnames.append("job.err")
+                        fnames.append(JOB_LOG_ERR)
                 except TypeError:
                     pass
                 fname_oks = {}
                 for fname in fnames:
-                    fname_oks[fname] = os.path.exists(self.get_task_job_log(
+                    fname_oks[fname] = os.path.exists(get_task_job_log(
                         schd_ctx.suite, point, name, submit_num, fname))
                 # All expected paths must exist to record a good attempt
                 log_ctx = SuiteProcContext((key1, submit_num), None)
@@ -586,7 +600,7 @@ class TaskEventsManager(object):
                         if not exist_ok:
                             log_ctx.err += " %s" % fname
                     self.event_timers[id_key].unset_waiting()
-                self.log_task_job_activity(
+                log_task_job_activity(
                     log_ctx, schd_ctx.suite, point, name, submit_num)
             except KeyError:
                 if cylc.flags.debug:
@@ -596,7 +610,7 @@ class TaskEventsManager(object):
         """Helper for process_message, handle a failed message."""
         if event_time is None:
             event_time = get_current_time_string()
-        itask.set_event_time('finished', event_time)
+        itask.set_summary_time('finished', event_time)
         self.suite_db_mgr.put_update_task_jobs(itask, {
             "run_status": 1,
             "time_run_exit": event_time,
@@ -605,22 +619,21 @@ class TaskEventsManager(object):
                 itask.try_timers[TASK_STATUS_RETRYING].next() is None):
             # No retry lined up: definitive failure.
             self.pflag = True
-            itask.state.reset_state(TASK_STATUS_FAILED)
-            self.setup_event_handlers(itask, "failed", message)
+            if itask.state.reset_state(TASK_STATUS_FAILED):
+                self.setup_event_handlers(itask, "failed", message)
             LOG.critical("job(%02d) %s" % (
                 itask.submit_num, "failed"), itask=itask)
         else:
             # There is a retry lined up
-            timeout_str = (
-                itask.try_timers[TASK_STATUS_RETRYING].timeout_as_str())
             delay_msg = "retrying in %s" % (
-                itask.try_timers[TASK_STATUS_RETRYING].delay_as_seconds())
-            msg = "failed, %s (after %s)" % (delay_msg, timeout_str)
+                itask.try_timers[TASK_STATUS_RETRYING].delay_timeout_as_str())
+            msg = "failed, %s" % (delay_msg)
             LOG.info("job(%02d) %s" % (itask.submit_num, msg), itask=itask)
             itask.summary['latest_message'] = msg
-            self.setup_event_handlers(
-                itask, "retry", "%s, %s" % (self.JOB_FAILED, delay_msg))
-            itask.state.reset_state(TASK_STATUS_RETRYING)
+            if itask.state.reset_state(TASK_STATUS_RETRYING):
+                self.setup_event_handlers(
+                    itask, "retry", "%s, %s" % (self.JOB_FAILED, delay_msg))
+        self._reset_job_timers(itask)
 
     def _process_message_started(self, itask, event_time):
         """Helper for process_message, handle a started message."""
@@ -628,31 +641,21 @@ class TaskEventsManager(object):
             itask.job_vacated = False
             LOG.warning("Vacated job restarted", itask=itask)
         self.pflag = True
-        itask.state.reset_state(TASK_STATUS_RUNNING)
-        itask.set_event_time('started', event_time)
+        if itask.state.reset_state(TASK_STATUS_RUNNING):
+            self.setup_event_handlers(itask, 'started', 'job started')
+        itask.set_summary_time('started', event_time)
+        self._reset_job_timers(itask)
         self.suite_db_mgr.put_update_task_jobs(itask, {
             "time_run": itask.summary['started_time_string']})
-        if itask.summary['execution_time_limit']:
-            execution_timeout = itask.summary['execution_time_limit']
-        else:
-            execution_timeout = self._get_events_conf(
-                itask, 'execution timeout')
-        try:
-            itask.timeout_timers[TASK_STATUS_RUNNING] = (
-                itask.summary['started_time'] + float(execution_timeout))
-        except (TypeError, ValueError):
-            itask.timeout_timers[TASK_STATUS_RUNNING] = None
 
         # submission was successful so reset submission try number
         if TASK_STATUS_SUBMIT_RETRYING in itask.try_timers:
             itask.try_timers[TASK_STATUS_SUBMIT_RETRYING].num = 0
-        self.setup_event_handlers(itask, 'started', 'job started')
-        self.set_poll_time(itask)
 
     def _process_message_succeeded(self, itask, event_time):
         """Helper for process_message, handle a succeeded message."""
         self.pflag = True
-        itask.set_event_time('finished', event_time)
+        itask.set_summary_time('finished', event_time)
         self.suite_db_mgr.put_update_task_jobs(itask, {
             "run_status": 0,
             "time_run_exit": event_time,
@@ -663,12 +666,18 @@ class TaskEventsManager(object):
                 itask.summary['finished_time'] -
                 itask.summary['started_time'])
         if not itask.state.outputs.all_completed():
-            message = "Succeeded with unreported outputs:"
+            msg = ""
             for output in itask.state.outputs.get_not_completed():
-                message += "\n  " + output
-            LOG.info(message, itask=itask)
-        itask.state.reset_state(TASK_STATUS_SUCCEEDED)
-        self.setup_event_handlers(itask, "succeeded", "job succeeded")
+                if output not in [TASK_OUTPUT_EXPIRED,
+                                  TASK_OUTPUT_SUBMIT_FAILED,
+                                  TASK_OUTPUT_FAILED]:
+                    msg += "\n  " + output
+            if msg:
+                LOG.info("Succeeded with outputs not completed: %s" % msg,
+                         itask=itask)
+        if itask.state.reset_state(TASK_STATUS_SUCCEEDED):
+            self.setup_event_handlers(itask, "succeeded", "job succeeded")
+        self._reset_job_timers(itask)
 
     def _process_message_submit_failed(self, itask, event_time):
         """Helper for process_message, handle a submit-failed message."""
@@ -683,26 +692,24 @@ class TaskEventsManager(object):
         if (TASK_STATUS_SUBMIT_RETRYING not in itask.try_timers or
                 itask.try_timers[TASK_STATUS_SUBMIT_RETRYING].next() is None):
             # No submission retry lined up: definitive failure.
-            itask.set_event_time('finished', event_time)
             self.pflag = True
             # See github #476.
-            self.setup_event_handlers(
-                itask, self.EVENT_SUBMIT_FAILED,
-                'job %s' % self.EVENT_SUBMIT_FAILED)
-            itask.state.reset_state(TASK_STATUS_SUBMIT_FAILED)
+            if itask.state.reset_state(TASK_STATUS_SUBMIT_FAILED):
+                self.setup_event_handlers(
+                    itask, self.EVENT_SUBMIT_FAILED,
+                    'job %s' % self.EVENT_SUBMIT_FAILED)
         else:
             # There is a submission retry lined up.
             timer = itask.try_timers[TASK_STATUS_SUBMIT_RETRYING]
-            timeout_str = timer.timeout_as_str()
-            delay_msg = "submit-retrying in %s" % timer.delay_as_seconds()
-            msg = "%s, %s (after %s)" % (
-                self.EVENT_SUBMIT_FAILED, delay_msg, timeout_str)
+            delay_msg = "submit-retrying in %s" % timer.delay_timeout_as_str()
+            msg = "%s, %s" % (self.EVENT_SUBMIT_FAILED, delay_msg)
             LOG.info("job(%02d) %s" % (itask.submit_num, msg), itask=itask)
             itask.summary['latest_message'] = msg
-            self.setup_event_handlers(
-                itask, self.EVENT_SUBMIT_RETRY,
-                "job %s, %s" % (self.EVENT_SUBMIT_FAILED, delay_msg))
-            itask.state.reset_state(TASK_STATUS_SUBMIT_RETRYING)
+            if itask.state.reset_state(TASK_STATUS_SUBMIT_RETRYING):
+                self.setup_event_handlers(
+                    itask, self.EVENT_SUBMIT_RETRY,
+                    "job %s, %s" % (self.EVENT_SUBMIT_FAILED, delay_msg))
+        self._reset_job_timers(itask)
 
     def _process_message_submitted(self, itask, event_time):
         """Helper for process_message, handle a submit-succeeded message."""
@@ -721,31 +728,26 @@ class TaskEventsManager(object):
 
         if itask.tdef.run_mode == 'simulation':
             # Simulate job execution at this point.
-            itask.set_event_time('started', event_time)
+            itask.set_summary_time('submitted', event_time)
+            itask.set_summary_time('started', event_time)
             itask.state.reset_state(TASK_STATUS_RUNNING)
             itask.state.outputs.set_completion(TASK_OUTPUT_STARTED, True)
             return
 
-        itask.set_event_time('submitted', event_time)
-        itask.set_event_time('started')
-        itask.set_event_time('finished')
+        itask.set_summary_time('submitted', event_time)
+        # Unset started and finished times in case of resubmission.
+        itask.set_summary_time('started')
+        itask.set_summary_time('finished')
         itask.summary['latest_message'] = TASK_OUTPUT_SUBMITTED
-        self.setup_event_handlers(
-            itask, TASK_OUTPUT_SUBMITTED, 'job submitted')
 
         self.pflag = True
         if itask.state.status == TASK_STATUS_READY:
-            # In rare occassions, the submit command of a batch system has sent
-            # the job to its server, and the server has started the job before
-            # the job submit command returns.
-            itask.state.reset_state(TASK_STATUS_SUBMITTED)
-            try:
-                itask.timeout_timers[TASK_STATUS_SUBMITTED] = (
-                    itask.summary['submitted_time'] +
-                    float(self._get_events_conf(itask, 'submission timeout')))
-            except (TypeError, ValueError):
-                itask.timeout_timers[TASK_STATUS_SUBMITTED] = None
-            self.set_poll_time(itask)
+            # The job started message can (rarely) come in before the submit
+            # command returns - in which case do not go back to 'submitted'.
+            if itask.state.reset_state(TASK_STATUS_SUBMITTED):
+                self.setup_event_handlers(
+                    itask, TASK_OUTPUT_SUBMITTED, 'job submitted')
+            self._reset_job_timers(itask)
 
     def _setup_job_logs_retrieval(self, itask, event):
         """Set up remote job logs retrieval.
@@ -825,13 +827,16 @@ class TaskEventsManager(object):
                 key1, str(itask.point), itask.tdef.name, itask.submit_num)
             if id_key in self.event_timers:
                 continue
+            # Note: user@host may not always be set for a submit number, e.g.
+            # on late event or if host select command fails. Use null string to
+            # prevent issues in this case.
+            user_at_host = itask.summary['job_hosts'].get(itask.submit_num, '')
+            if user_at_host and '@' not in user_at_host:
+                # (only has 'user@' on the front if user is not suite owner).
+                user_at_host = '%s@%s' % (get_user(), user_at_host)
             # Custom event handler can be a command template string
             # or a command that takes 4 arguments (classic interface)
             # Note quote() fails on None, need str(None).
-            user_at_host = itask.summary['job_hosts'][itask.submit_num]
-            if '@' not in user_at_host:
-                # (only has 'user@' on the front if user is not suite owner).
-                user_at_host = '%s@%s' % (get_user(), user_at_host)
             try:
                 handler_data = {
                     "event": quote(event),
@@ -887,3 +892,77 @@ class TaskEventsManager(object):
                         cmd,
                     ),
                     retry_delays))
+
+    def _reset_job_timers(self, itask):
+        """Set up poll timer and timeout for task."""
+        if itask.state.status not in TASK_STATUSES_ACTIVE:
+            # Reset, task not active
+            itask.timeout = None
+            itask.poll_timer = None
+            return
+        ctx = (itask.submit_num, itask.state.status)
+        if itask.poll_timer and itask.poll_timer.ctx == ctx:
+            return
+        # Set poll timer
+        # Set timeout
+        timeref = None  # reference time, submitted or started time
+        timeout = None  # timeout in setting
+        if itask.state.status == TASK_STATUS_RUNNING:
+            timeref = itask.summary['started_time']
+            timeout_key = 'execution timeout'
+            timeout = self._get_events_conf(itask, timeout_key)
+            delays = list(self.get_host_conf(
+                itask, 'execution polling intervals', skey='job',
+                default=[900]))  # Default 15 minute intervals
+            if itask.summary[self.KEY_EXECUTE_TIME_LIMIT]:
+                time_limit = itask.summary[self.KEY_EXECUTE_TIME_LIMIT]
+                try:
+                    host_conf = self.get_host_conf(itask, 'batch systems')
+                    batch_sys_conf = host_conf[itask.summary['batch_sys_name']]
+                except (TypeError, KeyError):
+                    batch_sys_conf = {}
+                time_limit_delays = batch_sys_conf.get(
+                    'execution time limit polling intervals', [60, 120, 420])
+                timeout = time_limit + sum(time_limit_delays)
+                # Remove execessive polling before time limit
+                while sum(delays) > time_limit:
+                    del delays[-1]
+                # But fill up the gap before time limit
+                if delays:
+                    size = int((time_limit - sum(delays)) / delays[-1])
+                    delays.extend([delays[-1]] * size)
+                time_limit_delays[0] += time_limit - sum(delays)
+                delays += time_limit_delays
+        else:  # if itask.state.status == TASK_STATUS_SUBMITTED:
+            timeref = itask.summary['submitted_time']
+            timeout_key = 'submission timeout'
+            timeout = self._get_events_conf(itask, timeout_key)
+            delays = list(self.get_host_conf(
+                itask, 'submission polling intervals', skey='job',
+                default=[900]))  # Default 15 minute intervals
+        try:
+            itask.timeout = timeref + float(timeout)
+            timeout_str = intvl_as_str(timeout)
+        except (TypeError, ValueError):
+            itask.timeout = None
+            timeout_str = None
+        itask.poll_timer = TaskActionTimer(ctx=ctx, delays=delays)
+        # Log timeout and polling schedule
+        message = 'health check settings: %s=%s' % (timeout_key, timeout_str)
+        # Attempt to group idenitical consecutive delays as N*DELAY,...
+        if itask.poll_timer.delays:
+            items = []  # [(number of item - 1, item), ...]
+            for delay in itask.poll_timer.delays:
+                if items and items[-1][1] == delay:
+                    items[-1][0] += 1
+                else:
+                    items.append([0, delay])
+            message += ', polling intervals='
+            for num, item in items:
+                if num:
+                    message += '%d*' % (num + 1)
+                message += '%s,' % intvl_as_str(item)
+            message += '...'
+        LOG.info(message, itask=itask)
+        # Set next poll time
+        self.check_poll_time(itask)
