@@ -1,4 +1,4 @@
-#!/usr/bin/env python
+#!/usr/bin/env python2
 
 # THIS FILE IS PART OF THE CYLC SUITE ENGINE.
 # Copyright (C) 2008-2018 NIWA
@@ -26,7 +26,8 @@ This module provides logic to:
 * Prepare task jobs poll/kill, and manage the callbacks.
 """
 
-from logging import CRITICAL, INFO, WARNING
+import json
+from logging import DEBUG, CRITICAL, INFO, WARNING
 import os
 from shutil import rmtree
 from time import time
@@ -34,18 +35,21 @@ import traceback
 
 from parsec.util import pdeepcopy, poverride
 
-from cylc.batch_sys_manager import BatchSysManager
-from cylc.cfgspec.globalcfg import GLOBAL_CFG
+from cylc.batch_sys_manager import BatchSysManager, JobPollContext
+from cylc.cfgspec.glbl_cfg import glbl_cfg
 from cylc.envvar import expandvars
 import cylc.flags
 from cylc.hostuserutil import is_remote_host, is_remote_user
 from cylc.job_file import JobFileWriter
+from cylc.task_job_logs import (
+    JOB_LOG_JOB, get_task_job_log, get_task_job_job_log,
+    get_task_job_activity_log, get_task_job_id, NN)
 from cylc.mkdir_p import mkdir_p
 from cylc.mp_pool import SuiteProcPool, SuiteProcContext
 from cylc.suite_logging import LOG
 from cylc.task_action_timer import TaskActionTimer
-from cylc.task_events_mgr import TaskEventsManager
-from cylc.task_message import TaskMessage
+from cylc.task_events_mgr import TaskEventsManager, log_task_job_activity
+from cylc.task_message import FAIL_MESSAGE_PREFIX
 from cylc.task_outputs import (
     TASK_OUTPUT_SUBMITTED, TASK_OUTPUT_STARTED, TASK_OUTPUT_SUCCEEDED,
     TASK_OUTPUT_FAILED)
@@ -55,8 +59,7 @@ from cylc.task_state import (
     TASK_STATUSES_ACTIVE, TASK_STATUS_READY, TASK_STATUS_SUBMITTED,
     TASK_STATUS_RUNNING, TASK_STATUS_SUCCEEDED, TASK_STATUS_FAILED,
     TASK_STATUS_SUBMIT_RETRYING, TASK_STATUS_RETRYING)
-from cylc.wallclock import (
-    get_current_time_string, get_seconds_as_interval_string)
+from cylc.wallclock import get_current_time_string
 
 
 class TaskJobManager(object):
@@ -71,13 +74,12 @@ class TaskJobManager(object):
     * Remove suite contact files on job hosts.
     """
 
-    JOB_FILE_BASE = BatchSysManager.JOB_FILE_BASE
     JOBS_KILL = 'jobs-kill'
     JOBS_POLL = 'jobs-poll'
     JOBS_SUBMIT = SuiteProcPool.JOBS_SUBMIT
     REMOTE_SELECT_MSG = 'waiting for remote host selection'
     REMOTE_INIT_MSG = 'remote host initialising'
-    KEY_EXECUTE_TIME_LIMIT = 'execution_time_limit'
+    KEY_EXECUTE_TIME_LIMIT = TaskEventsManager.KEY_EXECUTE_TIME_LIMIT
 
     def __init__(self, suite, proc_pool, suite_db_mgr, suite_srv_files_mgr):
         self.suite = suite
@@ -98,9 +100,13 @@ class TaskJobManager(object):
         now = time()
         poll_tasks = set()
         for itask in task_pool.get_tasks():
-            if (self._check_timeout(itask, now) or
-                    self.task_events_mgr.set_poll_time(itask, now)):
+            if self.task_events_mgr.check_job_time(itask, now):
                 poll_tasks.add(itask)
+                if itask.poll_timer.delay is not None:
+                    LOG.info(
+                        'poll now, (next in %s)' % (
+                            itask.poll_timer.delay_timeout_as_str()),
+                        itask=itask)
         if poll_tasks:
             self.poll_task_jobs(suite, poll_tasks)
 
@@ -110,15 +116,15 @@ class TaskJobManager(object):
         If items is specified, kill active tasks matching given IDs.
 
         """
-        active_itasks = []
+        to_kill_tasks = []
         for itask in itasks:
             if itask.state.status in TASK_STATUSES_ACTIVE:
                 itask.state.set_held()
-                active_itasks.append(itask)
+                to_kill_tasks.append(itask)
             else:
                 LOG.warning('skipping %s: task not killable' % itask.identity)
         self._run_job_cmd(
-            self.JOBS_KILL, suite, active_itasks,
+            self.JOBS_KILL, suite, to_kill_tasks,
             self._kill_task_jobs_callback)
 
     def poll_task_jobs(self, suite, itasks, poll_succ=True, msg=None):
@@ -133,21 +139,23 @@ class TaskJobManager(object):
 
         _poll_task_job_callback() executes one specific job.
         """
-        poll_me = []
-        pollable = [TASK_STATUS_SUBMITTED, TASK_STATUS_RUNNING,
-                    TASK_STATUS_FAILED]
+        to_poll_tasks = []
+        pollable_statuses = set([
+            TASK_STATUS_SUBMITTED, TASK_STATUS_RUNNING, TASK_STATUS_FAILED])
+        if poll_succ:
+            pollable_statuses.add(TASK_STATUS_SUCCEEDED)
         for itask in itasks:
-            if itask.state.status in pollable or (
-                    itask.state.status == TASK_STATUS_SUCCEEDED and poll_succ):
-                poll_me.append(itask)
+            if itask.state.status in pollable_statuses:
+                to_poll_tasks.append(itask)
             else:
                 LOG.debug("skipping %s: not pollable, "
                           "or skipping 'succeeded' tasks" % itask.identity)
-        if poll_me:
+        if to_poll_tasks:
             if msg is not None:
                 LOG.info(msg)
             self._run_job_cmd(
-                self.JOBS_POLL, suite, poll_me, self._poll_task_jobs_callback)
+                self.JOBS_POLL, suite, to_poll_tasks,
+                self._poll_task_jobs_callback)
 
     def prep_submit_task_jobs(self, suite, itasks, dry_run=False,
                               check_syntax=True):
@@ -181,7 +189,7 @@ class TaskJobManager(object):
 
         This method uses prep_submit_task_job() as helper.
 
-        Return (list): list of tasks that attempted submission
+        Return (list): list of tasks that attempted submission.
         """
         if is_simulation:
             return self._simulation_submit_task_jobs(itasks)
@@ -235,7 +243,7 @@ class TaskJobManager(object):
                 # Set submit-failed for all affected tasks
                 for itask in itasks:
                     itask.local_job_file_path = None  # reset for retry
-                    self.task_events_mgr.log_task_job_activity(
+                    log_task_job_activity(
                         SuiteProcContext(
                             self.JOBS_SUBMIT,
                             '(init %s)' % owner_at_host,
@@ -244,13 +252,14 @@ class TaskJobManager(object):
                         suite, itask.point, itask.tdef.name)
                     self.task_events_mgr.process_message(
                         itask, CRITICAL,
-                        self.task_events_mgr.EVENT_SUBMIT_FAILED,
-                        self.poll_task_jobs)
+                        self.task_events_mgr.EVENT_SUBMIT_FAILED)
                 continue
             # Build the "cylc jobs-submit" command
             cmd = ['cylc', self.JOBS_SUBMIT]
             if cylc.flags.debug:
                 cmd.append('--debug')
+            if cylc.flags.utc:
+                cmd.append('--utc-mode')
             remote_mode = False
             kwargs = {}
             for key, value, test_func in [
@@ -263,17 +272,17 @@ class TaskJobManager(object):
             if remote_mode:
                 cmd.append('--remote-mode')
             cmd.append('--')
-            cmd.append(GLOBAL_CFG.get_derived_host_item(
+            cmd.append(glbl_cfg().get_derived_host_item(
                 suite, 'suite job log directory', host, owner))
             stdin_file_paths = []
             job_log_dirs = []
             for itask in sorted(itasks, key=lambda itask: itask.identity):
                 if remote_mode:
                     stdin_file_paths.append(
-                        self.task_events_mgr.get_task_job_log(
+                        get_task_job_job_log(
                             suite, itask.point, itask.tdef.name,
-                            itask.submit_num, self.JOB_FILE_BASE))
-                job_log_dirs.append(self.task_events_mgr.get_task_job_id(
+                            itask.submit_num))
+                job_log_dirs.append(get_task_job_id(
                     itask.point, itask.tdef.name, itask.submit_num))
                 # The job file is now (about to be) used: reset the file write
                 # flag so that subsequent manual retrigger will generate a new
@@ -294,44 +303,8 @@ class TaskJobManager(object):
                 self._submit_task_jobs_callback, [suite, itasks])
         return done_tasks
 
-    def _check_timeout(self, itask, now):
-        """Check/handle submission/execution timeouts."""
-        if itask.state.status == TASK_STATUS_RUNNING:
-            timer = itask.poll_timers.get(self.KEY_EXECUTE_TIME_LIMIT)
-            if timer is not None:
-                if not timer.is_timeout_set():
-                    timer.next()
-                if not timer.is_delay_done():
-                    # Don't poll
-                    return False
-                if timer.next() is not None:
-                    # Poll now, and more retries lined up
-                    return True
-                # No more retry lined up, can issue execution timeout event
-        if itask.state.status in itask.timeout_timers:
-            timeout = itask.timeout_timers[itask.state.status]
-            if timeout is None or now <= timeout:
-                return False
-            itask.timeout_timers[itask.state.status] = None
-            if (itask.state.status == TASK_STATUS_RUNNING and
-                    itask.summary['started_time'] is not None):
-                msg = 'job started %s ago, but has not finished' % (
-                    get_seconds_as_interval_string(
-                        timeout - itask.summary['started_time']))
-                event = 'execution timeout'
-            elif (itask.state.status == TASK_STATUS_SUBMITTED and
-                    itask.summary['submitted_time'] is not None):
-                msg = 'job submitted %s ago, but has not started' % (
-                    get_seconds_as_interval_string(
-                        timeout - itask.summary['submitted_time']))
-                event = 'submission timeout'
-            else:
-                return False
-            LOG.warning(msg, itask=itask)
-            self.task_events_mgr.setup_event_handlers(itask, event, msg)
-            return True
-
-    def _create_job_log_path(self, suite, itask):
+    @staticmethod
+    def _create_job_log_path(suite, itask):
         """Create job log directory for a task job, etc.
 
         Create local job directory, and NN symbolic link.
@@ -340,7 +313,7 @@ class TaskJobManager(object):
         Return a string in the form "POINT/NAME/SUBMIT_NUM".
 
         """
-        job_file_dir = self.task_events_mgr.get_task_job_log(
+        job_file_dir = get_task_job_log(
             suite, itask.point, itask.tdef.name, itask.submit_num)
         task_log_dir = os.path.dirname(job_file_dir)
         if itask.submit_num == 1:
@@ -350,7 +323,7 @@ class TaskJobManager(object):
                 pass
             else:
                 for name in names:
-                    if name not in ["01", self.task_events_mgr.NN]:
+                    if name not in ["01", NN]:
                         rmtree(
                             os.path.join(task_log_dir, name),
                             ignore_errors=True)
@@ -358,7 +331,7 @@ class TaskJobManager(object):
             rmtree(job_file_dir, ignore_errors=True)
 
         mkdir_p(job_file_dir)
-        target = os.path.join(task_log_dir, self.task_events_mgr.NN)
+        target = os.path.join(task_log_dir, NN)
         source = os.path.basename(job_file_dir)
         try:
             prev_source = os.readlink(target)
@@ -402,7 +375,8 @@ class TaskJobManager(object):
             script = "echo " + comstr + "\n" + comstr
         return pre_script, script, post_script
 
-    def _job_cmd_out_callback(self, suite, itask, cmd_ctx, line):
+    @staticmethod
+    def _job_cmd_out_callback(suite, itask, cmd_ctx, line):
         """Callback on job command STDOUT/STDERR."""
         if cmd_ctx.cmd_kwargs.get("host") and cmd_ctx.cmd_kwargs.get("user"):
             owner_at_host = "(%(user)s@%(host)s) " % cmd_ctx.cmd_kwargs
@@ -418,7 +392,7 @@ class TaskJobManager(object):
             pass
         else:
             line = "%s %s" % (timestamp, content)
-        job_activity_log = self.task_events_mgr.get_task_job_activity_log(
+        job_activity_log = get_task_job_activity_log(
             suite, itask.point, itask.tdef.name)
         try:
             with open(job_activity_log, "ab") as handle:
@@ -451,8 +425,7 @@ class TaskJobManager(object):
             ctx.ret_code = int(ctx.ret_code)
             if ctx.ret_code:
                 ctx.cmd = cmd_ctx.cmd  # print original command on failure
-        self.task_events_mgr.log_task_job_activity(
-            ctx, suite, itask.point, itask.tdef.name)
+        log_task_job_activity(ctx, suite, itask.point, itask.tdef.name)
         log_lvl = INFO
         log_msg = 'killed'
         if ctx.ret_code:  # non-zero exit status
@@ -461,16 +434,15 @@ class TaskJobManager(object):
             itask.state.kill_failed = True
         elif itask.state.status == TASK_STATUS_SUBMITTED:
             self.task_events_mgr.process_message(
-                itask, CRITICAL, "%s at %s" % (
-                    self.task_events_mgr.EVENT_SUBMIT_FAILED, ctx.timestamp),
-                self.poll_task_jobs)
+                itask, CRITICAL, self.task_events_mgr.EVENT_SUBMIT_FAILED,
+                ctx.timestamp)
             cylc.flags.iflag = True
         elif itask.state.status == TASK_STATUS_RUNNING:
             self.task_events_mgr.process_message(
-                itask, CRITICAL, TASK_OUTPUT_FAILED, self.poll_task_jobs)
+                itask, CRITICAL, TASK_OUTPUT_FAILED)
             cylc.flags.iflag = True
         else:
-            log_lvl = WARNING
+            log_lvl = DEBUG
             log_msg = (
                 'ignoring job kill result, unexpected task state: %s' %
                 itask.state.status)
@@ -541,65 +513,71 @@ class TaskJobManager(object):
         ctx.out = line
         ctx.ret_code = 0
 
-        items = line.split("|")
         # See cylc.batch_sys_manager.JobPollContext
         try:
-            (
-                batch_sys_exit_polled, run_status, run_signal,
-                time_submit_exit, time_run, time_run_exit
-            ) = items[4:10]
-        except IndexError:
-            itask.summary['latest_message'] = 'poll failed'
-            cylc.flags.iflag = True
-            ctx.cmd = cmd_ctx.cmd  # print original command on failure
-            return
+            job_log_dir, context = line.split('|')[1:4]
+            items = json.loads(context)
+        except ValueError:
+            # back compat for cylc 7.7.1 and previous
+            try:
+                values = line.split('|')
+                items = dict(  # done this way to ensure IndexError is raised
+                    (key, values[x]) for
+                    x, key in enumerate(JobPollContext.CONTEXT_ATTRIBUTES))
+                job_log_dir = items.pop('job_log_dir')
+            except (ValueError, IndexError):
+                itask.summary['latest_message'] = 'poll failed'
+                cylc.flags.iflag = True
+                ctx.cmd = cmd_ctx.cmd  # print original command on failure
+                return
         finally:
-            self.task_events_mgr.log_task_job_activity(
-                ctx, suite, itask.point, itask.tdef.name)
-        if run_status == "1" and run_signal in ["ERR", "EXIT"]:
+            log_task_job_activity(ctx, suite, itask.point, itask.tdef.name)
+
+        jp_ctx = JobPollContext(job_log_dir, **items)
+
+        flag = self.task_events_mgr.POLLED_FLAG
+        if jp_ctx.run_status == 1 and jp_ctx.run_signal in ["ERR", "EXIT"]:
             # Failed normally
             self.task_events_mgr.process_message(
-                itask, INFO, TASK_OUTPUT_FAILED, self.poll_task_jobs,
-                time_run_exit)
-        elif run_status == "1" and batch_sys_exit_polled == "1":
+                itask, INFO, TASK_OUTPUT_FAILED, jp_ctx.time_run_exit, flag)
+        elif jp_ctx.run_status == 1 and jp_ctx.batch_sys_exit_polled == 1:
             # Failed by a signal, and no longer in batch system
             self.task_events_mgr.process_message(
-                itask, INFO, TASK_OUTPUT_FAILED, self.poll_task_jobs,
-                time_run_exit)
+                itask, INFO, TASK_OUTPUT_FAILED, jp_ctx.time_run_exit, flag)
             self.task_events_mgr.process_message(
-                itask, INFO, TaskMessage.FAIL_MESSAGE_PREFIX + run_signal,
-                self.poll_task_jobs, time_run_exit)
-        elif run_status == "1":
+                itask, INFO, FAIL_MESSAGE_PREFIX + jp_ctx.run_signal,
+                jp_ctx.time_run_exit,
+                flag)
+        elif jp_ctx.run_status == 1:
             # The job has terminated, but is still managed by batch system.
             # Some batch system may restart a job in this state, so don't
             # mark as failed yet.
             self.task_events_mgr.process_message(
-                itask, INFO, TASK_OUTPUT_STARTED, self.poll_task_jobs,
-                time_run)
-        elif run_status == "0":
+                itask, INFO, TASK_OUTPUT_STARTED, jp_ctx.time_run, flag)
+        elif jp_ctx.run_status == 0:
             # The job succeeded
             self.task_events_mgr.process_message(
-                itask, INFO, TASK_OUTPUT_SUCCEEDED, self.poll_task_jobs,
-                time_run_exit)
-        elif time_run and batch_sys_exit_polled == "1":
+                itask, INFO, TASK_OUTPUT_SUCCEEDED, jp_ctx.time_run_exit,
+                flag)
+        elif jp_ctx.time_run and jp_ctx.batch_sys_exit_polled == 1:
             # The job has terminated without executing the error trap
             self.task_events_mgr.process_message(
-                itask, INFO, TASK_OUTPUT_FAILED, self.poll_task_jobs, "")
-        elif time_run:
+                itask, INFO, TASK_OUTPUT_FAILED, get_current_time_string(),
+                flag)
+        elif jp_ctx.time_run:
             # The job has started, and is still managed by batch system
             self.task_events_mgr.process_message(
-                itask, INFO, TASK_OUTPUT_STARTED, self.poll_task_jobs,
-                time_run)
-        elif batch_sys_exit_polled == "1":
+                itask, INFO, TASK_OUTPUT_STARTED, jp_ctx.time_run, flag)
+        elif jp_ctx.batch_sys_exit_polled == 1:
             # The job never ran, and no longer in batch system
             self.task_events_mgr.process_message(
                 itask, INFO, self.task_events_mgr.EVENT_SUBMIT_FAILED,
-                self.poll_task_jobs, time_submit_exit)
+                jp_ctx.time_submit_exit, flag)
         else:
             # The job never ran, and is in batch system
             self.task_events_mgr.process_message(
-                itask, INFO, TASK_STATUS_SUBMITTED, self.poll_task_jobs,
-                time_submit_exit)
+                itask, INFO, TASK_STATUS_SUBMITTED, jp_ctx.time_submit_exit,
+                flag)
 
     def _poll_task_job_message_callback(self, suite, itask, cmd_ctx, line):
         """Helper for _poll_task_jobs_callback, on message of one task job."""
@@ -613,9 +591,9 @@ class TaskJobManager(object):
         else:
             ctx.ret_code = 0
             self.task_events_mgr.process_message(
-                itask, severity, message, self.poll_task_jobs, event_time)
-        self.task_events_mgr.log_task_job_activity(
-            ctx, suite, itask.point, itask.tdef.name)
+                itask, severity, message, event_time,
+                self.task_events_mgr.POLLED_FLAG)
+        log_task_job_activity(ctx, suite, itask.point, itask.tdef.name)
 
     def _run_job_cmd(self, cmd_key, suite, itasks, callback):
         """Run job commands, e.g. poll, kill, etc.
@@ -640,11 +618,11 @@ class TaskJobManager(object):
             if is_remote_user(owner):
                 cmd.append("--user=%s" % (owner))
             cmd.append("--")
-            cmd.append(GLOBAL_CFG.get_derived_host_item(
+            cmd.append(glbl_cfg().get_derived_host_item(
                 suite, "suite job log directory", host, owner))
             job_log_dirs = []
             for itask in sorted(itasks, key=lambda itask: itask.identity):
-                job_log_dirs.append(self.task_events_mgr.get_task_job_id(
+                job_log_dirs.append(get_task_job_id(
                     itask.point, itask.tdef.name, itask.submit_num))
             cmd += job_log_dirs
             self.proc_pool.put_command(
@@ -680,7 +658,7 @@ class TaskJobManager(object):
             itask.summary[self.KEY_EXECUTE_TIME_LIMIT] = (
                 itask.tdef.rtconfig['job']['simulated run length'])
             self.task_events_mgr.process_message(
-                itask, INFO, TASK_OUTPUT_SUBMITTED, self.poll_task_jobs)
+                itask, INFO, TASK_OUTPUT_SUBMITTED)
         return itasks
 
     def _submit_task_jobs_callback(self, ctx, suite, itasks):
@@ -706,10 +684,9 @@ class TaskJobManager(object):
             ctx.ret_code = int(ctx.ret_code)
             if ctx.ret_code:
                 ctx.cmd = cmd_ctx.cmd  # print original command on failure
-        self.task_events_mgr.log_task_job_activity(
-            ctx, suite, itask.point, itask.tdef.name)
+        log_task_job_activity(ctx, suite, itask.point, itask.tdef.name)
 
-        if ctx.ret_code == SuiteProcPool.JOB_SKIPPED_FLAG:
+        if ctx.ret_code == SuiteProcPool.RET_CODE_SUITE_STOPPING:
             return
 
         try:
@@ -720,14 +697,11 @@ class TaskJobManager(object):
             itask.summary['submit_method_id'] = None
         if itask.summary['submit_method_id'] and ctx.ret_code == 0:
             self.task_events_mgr.process_message(
-                itask, INFO, '%s at %s' % (
-                    TASK_OUTPUT_SUBMITTED, ctx.timestamp),
-                self.poll_task_jobs)
+                itask, INFO, TASK_OUTPUT_SUBMITTED, ctx.timestamp)
         else:
             self.task_events_mgr.process_message(
-                itask, CRITICAL, '%s at %s' % (
-                    self.task_events_mgr.EVENT_SUBMIT_FAILED, ctx.timestamp),
-                self.poll_task_jobs)
+                itask, CRITICAL, self.task_events_mgr.EVENT_SUBMIT_FAILED,
+                ctx.timestamp)
 
     def _prep_submit_task_job(self, suite, itask, dry_run, check_syntax=True):
         """Prepare a task job submission.
@@ -756,6 +730,7 @@ class TaskJobManager(object):
             # Submit number not yet incremented
             itask.submit_num += 1
             itask.summary['submit_num'] = itask.submit_num
+            itask.summary['job_hosts'][itask.submit_num] = ''
             # Retry delays, needed for the try_num
             self._set_retry_timers(itask, rtconfig)
             self._prep_submit_task_job_error(
@@ -774,9 +749,8 @@ class TaskJobManager(object):
 
         try:
             job_conf = self._prep_submit_task_job_impl(suite, itask, rtconfig)
-            local_job_file_path = self.task_events_mgr.get_task_job_log(
-                suite, itask.point, itask.tdef.name, itask.submit_num,
-                self.JOB_FILE_BASE)
+            local_job_file_path = get_task_job_job_log(
+                suite, itask.point, itask.tdef.name, itask.submit_num)
             self.job_file_writer.write(local_job_file_path, job_conf,
                                        check_syntax=check_syntax)
         except StandardError as exc:
@@ -799,7 +773,7 @@ class TaskJobManager(object):
         LOG.debug("submit_num %s" % itask.submit_num)
         LOG.debug(traceback.format_exc())
         LOG.error(exc)
-        self.task_events_mgr.log_task_job_activity(
+        log_task_job_activity(
             SuiteProcContext(self.JOBS_SUBMIT, action, err=exc, ret_code=1),
             suite, itask.point, itask.tdef.name, submit_num=itask.submit_num)
         if not dry_run:
@@ -812,8 +786,7 @@ class TaskJobManager(object):
             })
             itask.is_manual_submit = False
             self.task_events_mgr.process_message(
-                itask, CRITICAL, self.task_events_mgr.EVENT_SUBMIT_FAILED,
-                self.poll_task_jobs)
+                itask, CRITICAL, self.task_events_mgr.EVENT_SUBMIT_FAILED)
 
     def _prep_submit_task_job_impl(self, suite, itask, rtconfig):
         """Helper for self._prep_submit_task_job."""
@@ -830,7 +803,7 @@ class TaskJobManager(object):
             itask.summary['logfiles'].append(expandvars(name))
         try:
             batch_sys_conf = self.task_events_mgr.get_host_conf(
-                itask, 'batch systems')[rtconfig['job']['batch system']]
+                itask, 'batch systems')[itask.summary['batch_sys_name']]
         except (TypeError, KeyError):
             batch_sys_conf = {}
         try:
@@ -838,39 +811,24 @@ class TaskJobManager(object):
                 rtconfig['job']['execution time limit'])
         except TypeError:
             pass
-        if itask.summary[self.KEY_EXECUTE_TIME_LIMIT]:
-            # Default = 1, 2 and 7 minutes intervals, roughly 1, 3 and 10
-            # minutes after time limit exceeded
-            itask.poll_timers[self.KEY_EXECUTE_TIME_LIMIT] = (
-                TaskActionTimer(delays=batch_sys_conf.get(
-                    'execution time limit polling intervals', [60, 120, 420])))
-        for label, key in [
-                ('submission polling intervals', TASK_STATUS_SUBMITTED),
-                ('execution polling intervals', TASK_STATUS_RUNNING)]:
-            if key in itask.poll_timers:
-                itask.poll_timers[key].reset()
-            else:
-                values = self.task_events_mgr.get_host_conf(
-                    itask, label, skey='job')
-                if values:
-                    itask.poll_timers[key] = TaskActionTimer(delays=values)
 
         scripts = self._get_job_scripts(itask, rtconfig)
 
         # Location of job file, etc
         self._create_job_log_path(suite, itask)
-        job_d = self.task_events_mgr.get_task_job_id(
+        job_d = get_task_job_id(
             itask.point, itask.tdef.name, itask.submit_num)
         job_file_path = os.path.join(
-            GLOBAL_CFG.get_derived_host_item(
+            glbl_cfg().get_derived_host_item(
                 suite, "suite job log directory",
                 itask.task_host, itask.task_owner),
-            job_d, self.JOB_FILE_BASE)
+            job_d, JOB_LOG_JOB)
         return {
             'batch_system_name': rtconfig['job']['batch system'],
             'batch_submit_command_template': (
                 rtconfig['job']['batch submit command template']),
             'batch_system_conf': batch_sys_conf,
+            'dependencies': itask.state.get_resolved_dependencies(),
             'directives': rtconfig['directives'],
             'environment': rtconfig['environment'],
             'execution_time_limit': itask.summary[self.KEY_EXECUTE_TIME_LIMIT],
@@ -893,5 +851,6 @@ class TaskJobManager(object):
             'suite_name': suite,
             'task_id': itask.identity,
             'try_num': itask.get_try_num(),
+            'uuid_str': self.task_remote_mgr.uuid_str,
             'work_d': rtconfig['work sub-directory'],
         }
