@@ -1,4 +1,4 @@
-#!/usr/bin/env python
+#!/usr/bin/env python2
 
 # THIS FILE IS PART OF THE CYLC SUITE ENGINE.
 # Copyright (C) 2008-2018 NIWA
@@ -60,6 +60,10 @@ class TaskOutputs(object):
     def __init__(self, tdef):
         self._by_message = {}
         self._by_trigger = {}
+        # Add standard outputs.
+        for output in _SORT_ORDERS:
+            self.add(output)
+        # Add custom message outputs.
         for trigger, message in tdef.outputs:
             self.add(message, trigger)
 
