@@ -1,4 +1,4 @@
-#!/usr/bin/env python
+#!/usr/bin/env python2
 
 # THIS FILE IS PART OF THE CYLC SUITE ENGINE.
 # Copyright (C) 2008-2018 NIWA
@@ -129,7 +129,7 @@ class config(object):
                 null = False
                 i = i.lstrip('[')
                 i = i.rstrip(']')
-                j = re.split('\]\[*', i)
+                j = re.split(r'\]\[*', i)
                 mkeys.append(j)
         if null:
             mkeys = [[]]
