@@ -1,4 +1,4 @@
-#!/usr/bin/env python
+#!/usr/bin/env python2
 
 # THIS FILE IS PART OF THE CYLC SUITE ENGINE.
 # Copyright (C) 2008-2018 NIWA
@@ -88,7 +88,10 @@ def printcfg(cfg, level=0, indent=0, prefix='', none_str='',
             elif isinstance(cfg_i, list):
                 value = listjoin(cfg_i, none_str)
             elif "\n" in str(cfg_i) and key:
-                value = '"""\n%s\n"""' % (cfg_i)
+                value = '"""\n'
+                for line in str(cfg_i).splitlines(True):
+                    value += spacer + " " * 4 + line
+                value += '\n' + spacer + '"""'
             else:
                 value = str(cfg_i)
             if value is not None:
